/* C Define for CH376        */
/* Website:  http://wch.cn   */
/* Email:    tech@wch.cn     */
/* Author:   W.ch 2008.10    */
/* V1.0 for CH376            */

#ifndef __CH376INC_H__
#define __CH376INC_H__

#ifdef __cplusplus
extern "C" {
#endif

/* ********************************************************************************************************************* */
/* Common types and constant definitions */

#ifndef		TRUE
#define		TRUE	1
#define		FALSE	0
#endif
#ifndef		NULL
#define		NULL	0
#endif

#ifndef UINT8
typedef unsigned char                UINT8;
#endif
#ifndef UINT16
typedef unsigned short               UINT16;
#endif
#ifndef UINT32
typedef unsigned long                UINT32;
#endif
#ifndef PUINT8
typedef unsigned char               *PUINT8;
#endif
#ifndef PUINT16
typedef unsigned short              *PUINT16;
#endif
#ifndef PUINT32
typedef unsigned long               *PUINT32;
#endif
#ifndef UINT8V
typedef unsigned char volatile       UINT8V;
#endif
#ifndef PUINT8V
typedef unsigned char volatile      *PUINT8V;
#endif

/* ********************************************************************************************************************* */
/* Hardware features */

#define	CH376_DAT_BLOCK_LEN	0x40			/* USB single packet, maximum length of data block, default buffer length */

/* ********************************************************************************************************************* */
/* Command code */
/* Some commands are compatible with CH375 chip, but the input data or output data may be partially different) */
/* A sequence of command operations consists of:
           A command code (for the serial port mode, two synchronization codes are required before the command code),
           Several input data (can be 0),
           Generate an interrupt notification or a number of output data (can be 0), choose one of the two, if there is an interrupt notification, there must be no output data, and if there is output data, no interrupt will be generated
        Only the CMD01_WR_REQ_DATA command is an exception, and the sequence includes: a command code, an output data, and several input data
    Command code naming rules: CMDxy_NAME
        Both x and y are numbers, x indicates the minimum number of input data (number of bytes), y indicates the minimum number of output data (number of bytes), if y is H, it indicates that an interrupt notification is generated,
        Some commands can read and write data blocks of 0 to more bytes, and the number of bytes of the data block itself is not included in the above x or y	   */
/* By default, this file will also provide a command code format compatible with the command code of the CH375 chip (that is, after removing x and y), if not needed, you can define _NO_CH375_COMPATIBLE_ to prohibit */

/* ********************************************************************************************************************* */
/* Main commands (manual 1), commonly used */

#define	CMD01_GET_IC_VER	0x01			/* Get chip and firmware version */
/* Output: version number (bit 7 is 0, bit 6 is 1, bit 5~bit 0 is the version number) */
/* CH376 returns the value of the version number as 041H, that is, the version number is 01H */

#define	CMD21_SET_BAUDRATE	0x02			/* Serial port mode: Set the serial port communication baud rate (the default baud rate after power-on or reset is 9600bps, selected by D4/D5/D6 pins) */
/* Input: baud rate division factor, baud rate division constant */
/* Output: Operation status (CMD_RET_SUCCESS or CMD_RET_ABORT, other values indicate that the operation is not completed) */

#define	CMD00_ENTER_SLEEP	0x03			/* Go to sleep */

#define	CMD00_RESET_ALL		0x05			/* Perform a hardware reset */

#define	CMD11_CHECK_EXIST	0x06			/* Test communication interface and working status */
/* Input: any data */
/* Output: bitwise inverse of the input data */

#define	CMD20_CHK_SUSPEND	0x0B			/* Device Mode: Set the way to check the USB bus suspend status */
/* Input: data 10H, check mode */
/* 00H=Do not check USB suspend, 04H=Check USB suspend every 50mS, 05H=Check USB suspend every 10mS */

#define	CMD20_SET_SDO_INT	0x0B			/* SPI interface mode: Set the interrupt mode of the SDO pin of SPI */
/* Input: data 16H, interrupt mode */
/* 10H=Prohibit SDO pin for interrupt output, when SCS chip selection is invalid, three-state output is prohibited, 90H=SDO pin is also used as interrupt request output when SCS chip selection is invalid */

#define	CMD14_GET_FILE_SIZE	0x0C			/* Host file mode: get the current file length */
/* Input: Data 68H */
/* Output: current file length (total length 32 bits, low byte first) */

#define	CMD50_SET_FILE_SIZE	0x0D			/* Host file mode: set the current file length */
/* Input: data 68H, current file length (total length 32 bits, low byte first) */

#define	CMD11_SET_USB_MODE	0x15			/* Set USB working mode */
/* Input: pattern code */
/*       00H=Disabled device mode, 01H=Enabled device mode and use external firmware mode (serial port not supported), 02H=Enabled device mode and use built-in firmware mode */
/*       03H=SD card host mode/host mode disabled, used to manage and access files in SD card */
/*       04H=Disabled host mode, 05H=Enabled host mode, 06H=Enabled host mode and automatically generate SOF packet, 07H=Enabled host mode and reset USB bus */
/* Output: Operation status (CMD_RET_SUCCESS or CMD_RET_ABORT, other values indicate that the operation is not completed) */

#define	CMD01_GET_STATUS	0x22			/* Get interrupt status and cancel interrupt request */
/* Output: interrupt status */

#define	CMD00_UNLOCK_USB	0x23			/* Device mode: release the current USB buffer */

#define	CMD01_RD_USB_DATA0	0x27			/* Read a block of data from the endpoint buffer of the current USB interrupt or the receive buffer of the host endpoint */
/* Output: length, stream */

#define	CMD01_RD_USB_DATA	0x28			/* 设read the data block from the endpoint buffer of the current USB interrupt, and release the buffer, which is equivalent to CMD01_RD_USB_DATA0 + CMD00_UNLOCK_USB */
/* Output: length, stream  */

#define	CMD10_WR_USB_DATA7	0x2B			/* Write a data block to the send buffer of USB endpoint 2 */
/* Input: length, stream */

#define	CMD10_WR_HOST_DATA	0x2C			/* Write a block of data to the send buffer of the USB host endpoint */
/* Input: length, stream */

#define	CMD01_WR_REQ_DATA	0x2D			/* Write the requested data block to the internal specified buffer */
/* Output: length */
/* Input: data stream */

#define	CMD20_WR_OFS_DATA	0x2E			/* Write a data block to the specified offset address of the internal buffer */
/* Input: offset, length, stream */

#define	CMD10_SET_FILE_NAME	0x2F			/* Host file mode: set the file name of the file to be operated on */
/* Input: A string terminated by 0 (with a length of no more than 14 characters including the terminator 0) */

/* ********************************************************************************************************************* */
/* Main command (Manual 1), commonly used, the following commands always generate an interrupt notification at the end of the operation, and always have no output data */

#define	CMD0H_DISK_CONNECT	0x30			/* Hosts file mode/SD card not supported: check if the disk is connected */
/* Output interrupt */

#define	CMD0H_DISK_MOUNT	0x31			/* Hosts file mode: initialize the disk and test if the disk is ready */
/* Output interrupt */

#define	CMD0H_FILE_OPEN		0x32			/* Host file mode: open a file or directory (folder), or enumerate files and directories (folder) */
/* Output interrupt */

#define	CMD0H_FILE_ENUM_GO	0x33			/* Hosts file mode: continue to enumerate files and directories (folders) */
/* Output interrupt */

#define	CMD0H_FILE_CREATE	0x34			/* Host file mode: Create a new file, if the file already exists, delete it first */
/* Output interrupt */

#define	CMD0H_FILE_ERASE	0x35			/* Host file mode: delete the file, if it is already opened, delete it directly, otherwise the file will be opened first and then deleted, the subdirectory must be opened first */
/* Output interrupt */

#define	CMD1H_FILE_CLOSE	0x36			/* Host file mode: close the currently opened file or directory (folder) */
/* Input: Whether to allow updating the file length */
/*          00H=prohibit update length, 01H=allow update length */
/* Output interrupt */

#define	CMD1H_DIR_INFO_READ	0x37			/* Hosts file mode: read directory information for a file */
/* Input: Specify the index number of the directory information structure to be read in the sector */
/*           The index number ranges from 00H to 0FH, and the index number 0FFH is the currently opened file */
/* Output interrupt */

#define	CMD0H_DIR_INFO_SAVE	0x38			/* Host file mode: directory information for saving files */
/* Output interrupt */

#define	CMD4H_BYTE_LOCATE	0x39			/* Host file mode: move the current file pointer in bytes */
/* Input: number of offset bytes (total length 32 bits, low byte first) */
/* Output interrupt */

#define	CMD2H_BYTE_READ		0x3A			/* Host file mode: read data blocks from the current position in bytes */
/* Input: the number of bytes requested to be read (total length 16 bits, low byte first) */
/* Output interrupt */

#define	CMD0H_BYTE_RD_GO	0x3B			/* Host file mode: continue byte read */
/* Output interrupt */

#define	CMD2H_BYTE_WRITE	0x3C			/* Host file mode: write data blocks to the current location in bytes */
/* Input: the number of bytes requested to be written (total length 16 bits, low byte first) */
/* Output interrupt */

#define	CMD0H_BYTE_WR_GO	0x3D			/* Host file mode: continue byte write */
/* Output interrupt */

#define	CMD0H_DISK_CAPACITY	0x3E			/* Host file mode: query disk physical capacity */
/* Output interrupt */

#define	CMD0H_DISK_QUERY	0x3F			/* Hosts file mode: query disk space information */
/* Output interrupt */

#define	CMD0H_DIR_CREATE	0x40			/* Host file mode: Create a new directory (folder) and open it, if the directory already exists, open it directly */
/* Output interrupt */

#define	CMD4H_SEC_LOCATE	0x4A			/* Host file mode: move the current file pointer in units of sectors */
/* Input: number of sectors requested to read */
/* Output interrupt */

#define	CMD1H_SEC_READ		0x4B			/* Host file mode/SD card not supported: read data block from current position in sector */
/* Input: number of sectors requested to read */
/* Output interrupt */

#define	CMD1H_SEC_WRITE		0x4C			/* Host file mode/SD card not supported: Write data block at the current position in units of sectors */
/* Input: Number of sectors requested to be written */
/* Output interrupt */

#define	CMD0H_DISK_BOC_CMD	0x50			/* Host mode/SD card not supported: Command to execute BulkOnly transfer protocol for USB storage */
/* Output interrupt */

#define	CMD5H_DISK_READ		0x54			/* Host mode/SD card not supported: read physical sector from USB memory */
/* Input: LBA physical sector address (total length 32 bits, low byte first), sector number (01H~FFH) */
/* Output interrupt */

#define	CMD0H_DISK_RD_GO	0x55			/* Host mode/SD card is not supported: continue to perform the physical sector read operation of the USB storage */
/* Output interrupt */

#define	CMD5H_DISK_WRITE	0x56			/* Host mode/SD card not supported: Write physical sectors to USB memory */
/* Input: LBA physical sector address (total length 32 bits, low byte first), sector number (01H~FFH) */
/* Output interrupt */

#define	CMD0H_DISK_WR_GO	0x57			/* Host mode/SD card not supported: Continue to perform physical sector write operation of USB storage */
/* Output interrupt */

/* ********************************************************************************************************************* */
/* Auxiliary command (manual 2), not commonly used or for compatibility with CH375 and CH372 */

#define	CMD10_SET_USB_SPEED	0x04			/* Set the USB bus speed, it will automatically restore to 12Mbps full speed every time CMD11_SET_USB_MODE sets the USB working mode */
/* Input: Bus Speed Code */
/*           00H=12Mbps full speed FullSpeed (default value), 01H=1.5Mbps (only modify frequency), 02H=1.5Mbps low speed LowSpeed */

#define	CMD11_GET_DEV_RATE	0x0A			/* Host mode: Get the data rate type of the currently connected USB device */
/* Input: Data 07H */
/* Output: data rate type */
/*           If bit 4 is 1, it is a 1.5Mbps low-speed USB device, otherwise it is a 12Mbps full-speed USB device */

#define	CMD11_GET_TOGGLE	0x0A			/* Get the synchronization status of the OUT transaction */
/* Input: Data 1AH */
/* Output: sync status */
/*           the OUT transaction is synchronized, otherwise the OUT transaction is not synchronized */

#define	CMD11_READ_VAR8		0x0A			/* Read the specified 8-bit filesystem variable */
/* Input: variable address */
/* Output: data */

/*#define	CMD11_GET_MAX_LUN	= CMD11_READ_VAR8( VAR_UDISK_LUN )*/	/* Host mode: Get the maximum and current logical unit number of the USB storage */

#define	CMD20_SET_RETRY		0x0B			/* Host mode: Set the number of retries for USB transaction operations */
/* Input: data 25H, number of retries */
/*           If bit 7 is 0, no retry when receiving NAK, when bit 7 is 1 and bit 6 is 0, infinite retry when receiving NAK, when bit 7 is 1, when bit 6 is 1, retry at most 3 seconds when receiving NAK, bit 5~bit 0 is the number of retries after timeout */

#define	CMD20_WRITE_VAR8	0x0B			/* Sets the specified 8-bit filesystem variable */
/*Input: variable address, data */

/*#define	CMD20_SET_DISK_LUN	= CMD20_WRITE_VAR8( VAR_UDISK_LUN )*/	/* Host mode: Set the current logical unit number of the USB storage */

#define	CMD14_READ_VAR32	0x0C			/* Read the specified 32-bit filesystem variable */
/* Input: variable address */
/* Output: data (total length 32 bits, low byte first) */

#define	CMD50_WRITE_VAR32	0x0D			/* Sets the specified 32-bit filesystem variable */
/* Input: variable address, data (total length 32 bits, low byte first) */

#define	CMD01_DELAY_100US	0x0F			/* Delay 100uS (serial port does not support) */
/* Output: output 0 during the delay period, and output non-zero at the end of the delay */

#define	CMD40_SET_USB_ID	0x12			/* Device mode: set USB manufacturer VID and product PID */
/* Input: Vendor ID low byte, Vendor ID high byte, Product ID low byte, Product ID high byte */

#define	CMD10_SET_USB_ADDR	0x13			/* Set USB address */
/* Input: address value */

#define	CMD01_TEST_CONNECT	0x16			/* Host mode/SD card not supported: Check the connection status of the USB device */
/* Output: status (USB_INT_CONNECT or USB_INT_DISCONNECT or USB_INT_USB_READY, other values indicate that the operation is not completed) */

#define	CMD00_ABORT_NAK		0x17			/* Host mode: Abandon the retry of the current NAK */

#define	CMD10_SET_ENDP2		0x18			/* Device mode (serial port not supported): Set the receiver of USB endpoint 0 */
/* Input: working method */
/*           If bit 7 is 1, bit 6 is the synchronous trigger bit, otherwise the synchronous trigger bit remains unchanged */
/*           Bit 3~bit 0 is the transaction response mode: 0000-Ready ACK, 1110-Busy NAK, 1111-Error STALL */

#define	CMD10_SET_ENDP3		0x19			/* Device mode (serial port not supported): Set the transmitter of USB endpoint 0 */
/* Input: working method */
/*           If bit 7 is 1, bit 6 is the synchronous trigger bit, otherwise the synchronous trigger bit remains unchanged */
/*           Bit 3~bit 0 is the transaction response mode: 0000~1000-Ready ACK, 1110-Busy NAK, 1111-Error STALL */

#define	CMD10_SET_ENDP4		0x1A			/* Device mode (serial port not supported): Set the receiver of USB endpoint 1 */
/* Input: working method */
/*           If bit 7 is 1, bit 6 is the synchronous trigger bit, otherwise the synchronous trigger bit remains unchanged */
/*           Bit 3~bit 0 is the transaction response mode: 0000-Ready ACK, 1110-Busy NAK, 1111-Error STALL */

#define	CMD10_SET_ENDP5		0x1B			/* Device mode (serial port not supported): Set the transmitter of USB endpoint 1 */
/* Input: working method */
/*           If bit 7 is 1, bit 6 is the synchronous trigger bit, otherwise the synchronous trigger bit remains unchanged */
/*           Bit 3~bit 0 is the transaction response mode: 0000~1000-Ready ACK, 1110-Busy NAK, 1111-Error STALL */

#define	CMD10_SET_ENDP6		0x1C			/* Set receiver for USB endpoint 2/host endpoint */
/* Input: working method */
/*           If bit 7 is 1, bit 6 is the synchronous trigger bit, otherwise the synchronous trigger bit remains unchanged */
/*           Bit 3~bit 0 is the transaction response mode: 0000-Ready ACK, 1101-Ready but does not return ACK, 1110-Busy NAK, 1111-Error STALL */

#define	CMD10_SET_ENDP7		0x1D			/* Sets the transmitter of the USB endpoint 2/host endpoint */
/* Input: working method */
/*           If bit 7 is 1, bit 6 is the synchronous trigger bit, otherwise the synchronous trigger bit remains unchanged */
/*           Bit 3~bit 0 is the transaction response mode: 0000-Ready ACK, 1101-Ready but no response, 1110-Busy NAK, 1111-Error STALL */

#define	CMD00_DIRTY_BUFFER	0x25			/* Host file mode: clear internal disk and file buffers */

#define	CMD10_WR_USB_DATA3	0x29			/* Device mode (serial port does not support): Write data block to the send buffer of USB endpoint 0 */
/* Input: length, stream */

#define	CMD10_WR_USB_DATA5	0x2A			/* Device mode (serial port not supported): Write data block to the send buffer of USB endpoint 1 */
/* Input: length, stream */

/* ********************************************************************************************************************* */
/* Auxiliary commands (manual 2), not commonly used or for compatibility with CH375 and CH372, the following commands always generate an interrupt notification at the end of the operation, and always have no output data */

#define	CMD1H_CLR_STALL		0x41			/* Host Mode: Control Transfer - Clear Endpoint Errors */
/* Input: endpoint number */
/* Output interrupt */

#define	CMD1H_SET_ADDRESS	0x45			/* Host mode: control transfer - set USB address */
/* Input: address value */
/* Output interrupt */

#define	CMD1H_GET_DESCR		0x46			/* Host Mode: Control Transfer - Get Descriptor */
/* Input: descriptor type */
/* Output interrupt */

#define	CMD1H_SET_CONFIG	0x49			/* Host Mode: Control Transfer - Set USB Configuration */
/* Input: Configuration value */
/* Output interrupt */

#define	CMD0H_AUTO_SETUP	0x4D			/* Host mode/SD card not supported: Automatically configure USB devices */
/* Output interrupt */

#define	CMD2H_ISSUE_TKN_X	0x4E			/* Host mode: issue synchronization token, execute transaction, this command can replace CMD10_SET_ENDP6/CMD10_SET_ENDP7 + CMD1H_ISSUE_TOKEN */
/* Input: synchronization flags, transaction attributes */
/*           Bit 7 of the synchronization flag is the synchronization trigger bit of the host endpoint IN, bit 6 is the synchronization trigger bit of the host endpoint OUT, and bits 5~0 must be 0 */
/*           The lower 4 bits of the transaction attribute are the token, and the upper 4 bits are the endpoint number */
/* Output interrupt */

#define	CMD1H_ISSUE_TOKEN	0x4F			/* Host mode: issue a token, execute a transaction, it is recommended to use the CMD2H_ISSUE_TKN_X command */
/* Input: transaction attributes */
/*           The lower 4 bits are the token, and the upper 4 bits are the endpoint number */
/* Output interrupt */

#define	CMD0H_DISK_INIT		0x51			/* Host method/SD card not supported: Initialize USB storage */
/* Output interrupt */

#define	CMD0H_DISK_RESET	0x52			/* Host mode/SD card not supported: control transfer-reset USB memory */
/* Output interrupt */

#define	CMD0H_DISK_SIZE		0x53			/* Host mode/SD card not supported: Get the capacity of USB storage */
/* Output interrupt */

#define	CMD0H_DISK_INQUIRY	0x58			/* Host mode/SD card not supported: Query the characteristics of USB storage */
/* Output interrupt */

#define	CMD0H_DISK_READY	0x59			/* Host mode/SD card not supported: Check USB storage ready */
/* Output interrupt */

#define	CMD0H_DISK_R_SENSE	0x5A			/* Host mode/SD card not supported: Check USB storage for errors */
/* Output interrupt */

#define	CMD0H_RD_DISK_SEC	0x5B			/* Host file mode: read a sector of data from the disk into the internal buffer */
/* Output interrupt */

#define	CMD0H_WR_DISK_SEC	0x5C			/* Host file mode: write a sector of data from the internal buffer to disk */
/* Output interrupt */

#define	CMD0H_DISK_MAX_LUN	0x5D			/* Host mode: control transfer - get the maximum logical unit number of the USB memory */
/* Output interrupt */

/* ********************************************************************************************************************* */
/* The following definitions are only for compatibility with the command name format in the INCLUDE file of CH375 */

#ifndef	_NO_CH375_COMPATIBLE_
#define	CMD_GET_IC_VER		CMD01_GET_IC_VER
#define	CMD_SET_BAUDRATE	CMD21_SET_BAUDRATE
#define	CMD_ENTER_SLEEP		CMD00_ENTER_SLEEP
#define	CMD_RESET_ALL		CMD00_RESET_ALL
#define	CMD_CHECK_EXIST		CMD11_CHECK_EXIST
#define	CMD_CHK_SUSPEND		CMD20_CHK_SUSPEND
#define	CMD_SET_SDO_INT		CMD20_SET_SDO_INT
#define	CMD_GET_FILE_SIZE	CMD14_GET_FILE_SIZE
#define	CMD_SET_FILE_SIZE	CMD50_SET_FILE_SIZE
#define	CMD_SET_USB_MODE	CMD11_SET_USB_MODE
#define	CMD_GET_STATUS		CMD01_GET_STATUS
#define	CMD_UNLOCK_USB		CMD00_UNLOCK_USB
#define	CMD_RD_USB_DATA0	CMD01_RD_USB_DATA0
#define	CMD_RD_USB_DATA		CMD01_RD_USB_DATA
#define	CMD_WR_USB_DATA7	CMD10_WR_USB_DATA7
#define	CMD_WR_HOST_DATA	CMD10_WR_HOST_DATA
#define	CMD_WR_REQ_DATA		CMD01_WR_REQ_DATA
#define	CMD_WR_OFS_DATA		CMD20_WR_OFS_DATA
#define	CMD_SET_FILE_NAME	CMD10_SET_FILE_NAME
#define	CMD_DISK_CONNECT	CMD0H_DISK_CONNECT
#define	CMD_DISK_MOUNT		CMD0H_DISK_MOUNT
#define	CMD_FILE_OPEN		CMD0H_FILE_OPEN
#define	CMD_FILE_ENUM_GO	CMD0H_FILE_ENUM_GO
#define	CMD_FILE_CREATE		CMD0H_FILE_CREATE
#define	CMD_FILE_ERASE		CMD0H_FILE_ERASE
#define	CMD_FILE_CLOSE		CMD1H_FILE_CLOSE
#define	CMD_DIR_INFO_READ	CMD1H_DIR_INFO_READ
#define	CMD_DIR_INFO_SAVE	CMD0H_DIR_INFO_SAVE
#define	CMD_BYTE_LOCATE		CMD4H_BYTE_LOCATE
#define	CMD_BYTE_READ		CMD2H_BYTE_READ
#define	CMD_BYTE_RD_GO		CMD0H_BYTE_RD_GO
#define	CMD_BYTE_WRITE		CMD2H_BYTE_WRITE
#define	CMD_BYTE_WR_GO		CMD0H_BYTE_WR_GO
#define	CMD_DISK_CAPACITY	CMD0H_DISK_CAPACITY
#define	CMD_DISK_QUERY		CMD0H_DISK_QUERY
#define	CMD_DIR_CREATE		CMD0H_DIR_CREATE
#define	CMD_SEC_LOCATE		CMD4H_SEC_LOCATE
#define	CMD_SEC_READ		CMD1H_SEC_READ
#define	CMD_SEC_WRITE		CMD1H_SEC_WRITE
#define	CMD_DISK_BOC_CMD	CMD0H_DISK_BOC_CMD
#define	CMD_DISK_READ		CMD5H_DISK_READ
#define	CMD_DISK_RD_GO		CMD0H_DISK_RD_GO
#define	CMD_DISK_WRITE		CMD5H_DISK_WRITE
#define	CMD_DISK_WR_GO		CMD0H_DISK_WR_GO
#define	CMD_SET_USB_SPEED	CMD10_SET_USB_SPEED
#define	CMD_GET_DEV_RATE	CMD11_GET_DEV_RATE
#define	CMD_GET_TOGGLE		CMD11_GET_TOGGLE
#define	CMD_READ_VAR8		CMD11_READ_VAR8
#define	CMD_SET_RETRY		CMD20_SET_RETRY
#define	CMD_WRITE_VAR8		CMD20_WRITE_VAR8
#define	CMD_READ_VAR32		CMD14_READ_VAR32
#define	CMD_WRITE_VAR32		CMD50_WRITE_VAR32
#define	CMD_DELAY_100US		CMD01_DELAY_100US
#define	CMD_SET_USB_ID		CMD40_SET_USB_ID
#define	CMD_SET_USB_ADDR	CMD10_SET_USB_ADDR
#define	CMD_TEST_CONNECT	CMD01_TEST_CONNECT
#define	CMD_ABORT_NAK		CMD00_ABORT_NAK
#define	CMD_SET_ENDP2		CMD10_SET_ENDP2
#define	CMD_SET_ENDP3		CMD10_SET_ENDP3
#define	CMD_SET_ENDP4		CMD10_SET_ENDP4
#define	CMD_SET_ENDP5		CMD10_SET_ENDP5
#define	CMD_SET_ENDP6		CMD10_SET_ENDP6
#define	CMD_SET_ENDP7		CMD10_SET_ENDP7
#define	CMD_DIRTY_BUFFER	CMD00_DIRTY_BUFFER
#define	CMD_WR_USB_DATA3	CMD10_WR_USB_DATA3
#define	CMD_WR_USB_DATA5	CMD10_WR_USB_DATA5
#define	CMD_CLR_STALL		CMD1H_CLR_STALL
#define	CMD_SET_ADDRESS		CMD1H_SET_ADDRESS
#define	CMD_GET_DESCR		CMD1H_GET_DESCR
#define	CMD_SET_CONFIG		CMD1H_SET_CONFIG
#define	CMD_AUTO_SETUP		CMD0H_AUTO_SETUP
#define	CMD_ISSUE_TKN_X		CMD2H_ISSUE_TKN_X
#define	CMD_ISSUE_TOKEN		CMD1H_ISSUE_TOKEN
#define	CMD_DISK_INIT		CMD0H_DISK_INIT
#define	CMD_DISK_RESET		CMD0H_DISK_RESET
#define	CMD_DISK_SIZE		CMD0H_DISK_SIZE
#define	CMD_DISK_INQUIRY	CMD0H_DISK_INQUIRY
#define	CMD_DISK_READY		CMD0H_DISK_READY
#define	CMD_DISK_R_SENSE	CMD0H_DISK_R_SENSE
#define	CMD_RD_DISK_SEC		CMD0H_RD_DISK_SEC
#define	CMD_WR_DISK_SEC		CMD0H_WR_DISK_SEC
#define	CMD_DISK_MAX_LUN	CMD0H_DISK_MAX_LUN
#endif

/* ********************************************************************************************************************* */
/* Parallel port mode, bit definition of status port (read command port) */
#ifndef	PARA_STATE_INTB
#define	PARA_STATE_INTB		0x80			/* Bit 7 of the status port in parallel mode: interrupt flag, active low */
#define	PARA_STATE_BUSY		0x10			/* Bit 4 of the status port in parallel port mode: busy flag, high effective */
#endif

/* ********************************************************************************************************************* */
/* Serial port mode, the boot synchronization code before the operation command */
#ifndef	SER_CMD_TIMEOUT
#define	SER_CMD_TIMEOUT		32				/* Serial port command timeout time, the unit is mS, the interval between synchronization codes and between synchronization codes and command codes should be as short as possible, and the processing method after timeout is to discard */
#define	SER_SYNC_CODE1		0x57			/* The first serial port synchronization code to start the operation */
#define	SER_SYNC_CODE2		0xAB			/* The second serial port synchronization code to start the operation */
#endif

/* ********************************************************************************************************************* */
/* Operating state */

#ifndef	CMD_RET_SUCCESS
#define	CMD_RET_SUCCESS		0x51			/* The command operation succeeded */
#define	CMD_RET_ABORT		0x5F			/* The command operation failed */
#endif

/* ********************************************************************************************************************* */
/* USB interrupt status */

#ifndef	USB_INT_EP0_SETUP

/* The following status codes are special event interrupts. If the USB bus suspend check is enabled through CMD20_CHK_SUSPEND, then the interrupt status of USB bus suspend and sleep wakeup must be handled */
#define	USB_INT_USB_SUSPEND	0x05			/* USB bus suspend event */
#define	USB_INT_WAKE_UP		0x06			/* Woke up from sleep event */

/* The following status code 0XH is used for USB device mode */
/*   In the built-in firmware mode, only need to deal with: USB_INT_EP1_OUT, USB_INT_EP1_IN, USB_INT_EP2_OUT, USB_INT_EP2_IN */
/*   Bit 7-bit 4 is 0000 */
/*   Bit 3-bit 2 indicates the current transaction, 00=OUT, 10=IN, 11=SETUP */
/*   Bit 1-bit 0 indicates the current endpoint, 00=endpoint 0, 01=endpoint 1, 10=endpoint 2, 11=USB bus reset */
#define	USB_INT_EP0_SETUP	0x0C			/* SETUP for USB endpoint 0 */
#define	USB_INT_EP0_OUT		0x00			/* OUT of USB endpoint 0 */
#define	USB_INT_EP0_IN		0x08			/* IN of USB endpoint 0 */
#define	USB_INT_EP1_OUT		0x01			/* OUT of USB endpoint 1 */
#define	USB_INT_EP1_IN		0x09			/* IN of USB endpoint 1 */
#define	USB_INT_EP2_OUT		0x02			/* OUT of USB endpoint 2 */
#define	USB_INT_EP2_IN		0x0A			/* IN of USB endpoint 2 */
/* USB_INT_BUS_RESET	0x0000XX11B */		/* USB bus reset */
#define	USB_INT_BUS_RESET1	0x03			/* USB bus reset */
#define	USB_INT_BUS_RESET2	0x07			/* USB bus reset */
#define	USB_INT_BUS_RESET3	0x0B			/* USB bus reset */
#define	USB_INT_BUS_RESET4	0x0F			/* USB bus reset */

#endif

/* The following status codes 2XH-3XH are used for communication failure codes in USB host mode */
/*   Bit 7-bit 6 is 00 */
/*   Bit 5 is 1 */
/*   Bit 4 indicates whether the currently received packet is synchronous */
/*   Bit 3-bit 0 indicates the response of the USB device when the communication fails: 0010=ACK, 1010=NAK, 1110=STALL, 0011=DATA0, 1011=DATA1, XX00=timeout */
/* USB_INT_RET_ACK	0x001X0010B */			/* Error: ACK returned for IN transaction */
/* USB_INT_RET_NAK	0x001X1010B */			/* Error: NAK returned */
/* USB_INT_RET_STALL	0x001X1110B */		/* Error: return STALL */
/* USB_INT_RET_DATA0	0x001X0011B */		/* Error: DATA0 returned for OUT/SETUP transaction */
/* USB_INT_RET_DATA1	0x001X1011B */		/* Error: DATA1 returned for OUT/SETUP transaction */
/* USB_INT_RET_TOUT	0x001XXX00B */			/* Error: Return timed out */
/* USB_INT_RET_TOGX	0x0010X011B */			/* ERROR: Return data out of sync for IN transaction */
/* USB_INT_RET_PID	0x001XXXXXB */			/* Error: undefined */

/* The following status code 1XH is used for the operation status code of USB host mode */
#ifndef	USB_INT_SUCCESS
#define	USB_INT_SUCCESS		0x14			/* USB transaction or transfer operation succeeded */
#define	USB_INT_CONNECT		0x15			/* A USB device connection event is detected, which may be a new connection or reconnection after disconnection */
#define	USB_INT_DISCONNECT	0x16			/* USB device disconnect event detected */
#define	USB_INT_BUF_OVER	0x17			/* The data transmitted by USB is wrong or too much data buffer overflow */
#define	USB_INT_USB_READY	0x18			/* The USB device has been initialized (USB address has been assigned) */
#define	USB_INT_DISK_READ	0x1D			/* USB memory request data read */
#define	USB_INT_DISK_WRITE	0x1E			/* USB memory request data write */
#define	USB_INT_DISK_ERR	0x1F			/* USB memory operation failed */
#endif

/* The following status codes are used for file system error codes in host file mode */
#ifndef	ERR_DISK_DISCON
#define	ERR_DISK_DISCON		0x82			/* The disk is not connected, maybe the disk has been disconnected*/
#define	ERR_LARGE_SECTOR	0x84			/* Disk sector is too large, only supports 512 bytes per sector */
#define	ERR_TYPE_ERROR		0x92			/* The disk partition type is not supported, only FAT12/FAT16/BigDOS/FAT32 is supported, and it needs to be repartitioned by the disk management tool */
#define	ERR_BPB_ERROR		0xA1			/* The disk has not been formatted, or the parameters are wrong, and it needs to be reformatted by WINDOWS with default parameters */
#define	ERR_DISK_FULL		0xB1			/* The disk file is too full, the remaining space is too little or there is no space, and disk defragmentation is required */
#define	ERR_FDT_OVER		0xB2			/* There are too many files in the directory (folder), and there are no free directory entries. The number of files in the FAT12/FAT16 root directory should be less than 512, and disk defragmentation is required */
#define	ERR_FILE_CLOSE		0xB4			/* The file has been closed, if you need to use it, you should reopen the file */
#define	ERR_OPEN_DIR		0x41			/* The directory (folder) of the specified path is opened */
#define	ERR_MISS_FILE		0x42			/* The file at the specified path could not be found, maybe the file name is wrong */
#define	ERR_FOUND_NAME		0x43			/* A matching file name was found, or a directory (folder) was requested to be opened but the file was actually opened */
/* The following file system error codes are used for file system subroutines */
#define	ERR_MISS_DIR		0xB3			/* A subdirectory (folder) of the specified path was not found, the directory name may be wrong */
#define	ERR_LONG_BUF_OVER	0x48			/* Long file buffer overflow */
#define	ERR_LONG_NAME_ERR	0x49			/* The short file name does not have a corresponding long file name or the long file name is wrong */
#define	ERR_NAME_EXIST		0x4A			/* A short file with the same name already exists, it is recommended to regenerate another short file name */
#endif

/* ********************************************************************************************************************* */
/* The following status codes are used for disk and file status in host file mode, VAR_DISK_STATUS */
#ifndef	DEF_DISK_UNKNOWN
#define	DEF_DISK_UNKNOWN	0x00			/* Not yet initialized, unknown state */
#define	DEF_DISK_DISCONN	0x01			/* The disk is not connected or has been disconnected */
#define	DEF_DISK_CONNECT	0x02			/* The disk is connected but has not been initialized or is not recognized*/
#define	DEF_DISK_MOUNTED	0x03			/* The disk has been initialized successfully, but the file system has not been analyzed or the file system does not support */
#define	DEF_DISK_READY		0x10			/* The file system of the disk has been analyzed and is able to support */
#define	DEF_DISK_OPEN_ROOT	0x12			/* The root directory has been opened and must be closed after use. Note that the root directory of FAT12/FAT16 is a fixed length */
#define	DEF_DISK_OPEN_DIR	0x13			/* A subdirectory (folder) has been opened*/
#define	DEF_DISK_OPEN_FILE	0x14			/* Already opened the file */
#endif

/* ********************************************************************************************************************* */
/* File System Common Definitions */

#ifndef	DEF_SECTOR_SIZE
#define	DEF_SECTOR_SIZE		512				/* The default physical sector size of the U disk or SD card */
#endif

#ifndef	DEF_WILDCARD_CHAR
#define	DEF_WILDCARD_CHAR	0x2A			/* Wildcard '*' for pathnames */
#define	DEF_SEPAR_CHAR1		0x5C			/* Pathname separator '\' */
#define	DEF_SEPAR_CHAR2		0x2F			/* Pathname separator '/'*/
#define	DEF_FILE_YEAR		2004			/* Default file date: 2004 */
#define	DEF_FILE_MONTH		1				/* Default file date: January */
#define	DEF_FILE_DATE		1				/* Default file date: 1st */
#endif

#ifndef	ATTR_DIRECTORY

/* File directory information in the FAT data area */
typedef struct _FAT_DIR_INFO {
	UINT8	DIR_Name[11];					/* 00H, file name, 11 bytes in total, fill in blanks */
	UINT8	DIR_Attr;						/* 0BH, file attributes, refer to the following instructions */
	UINT8	DIR_NTRes;						/* 0CH */
	UINT8	DIR_CrtTimeTenth;				/* 0DH,the time of file creation, counted in units of 0.1 seconds */
	UINT16	DIR_CrtTime;					/* 0EH,the time the file was created */
	UINT16	DIR_CrtDate;					/* 10H,the date the file was created */
	UINT16	DIR_LstAccDate;					/* 12H,the date of the last access operation */
	UINT16	DIR_FstClusHI;					/* 14H */
	UINT16	DIR_WrtTime;					/* 16H,file modification time, refer to the previous macro MAKE_FILE_TIME */
	UINT16	DIR_WrtDate;					/* 18H,file modification date, refer to the previous macro MAKE_FILE_DATE */
	UINT16	DIR_FstClusLO;					/* 1AH */
	UINT32	DIR_FileSize;					/* 1CH,file length */
} FAT_DIR_INFO, *P_FAT_DIR_INFO;			/* 20H */

/* File properties */
#define ATTR_READ_ONLY			0x01		/* File is read-only */
#define ATTR_HIDDEN				0x02		/* File is implicit attribute */
#define ATTR_SYSTEM				0x04		/* File as system property */
#define ATTR_VOLUME_ID			0x08		/* Label */
#define ATTR_DIRECTORY			0x10		/* Subdirectories (folders) */
#define ATTR_ARCHIVE			0x20		/* File as archive attribute */
#define ATTR_LONG_NAME			( ATTR_READ_ONLY | ATTR_HIDDEN | ATTR_SYSTEM | ATTR_VOLUME_ID )	/* Long file name attribute */
#define ATTR_LONG_NAME_MASK		( ATTR_LONG_NAME | ATTR_DIRECTORY | ATTR_ARCHIVE )
/* File attribute UINT8 */
/* bit0 bit1 bit2 bit3 bit4 bit5 bit6 bit7 */
/*  Read only, Hide， System， Label， Table of contents， Archive， Undefined*/
/* File time UINT16 */
/* Time = (Hour<<11) + (Minute<<5) + (Second>>1) */
#define MAKE_FILE_TIME( h, m, s )	( (h<<11) + (m<<5) + (s>>1) )	/* Generate file time data of the specified hour, minute, and second */
/* File data UINT16 */
/* Date = ((Year-1980)<<9) + (Month<<5) + Day */
#define MAKE_FILE_DATE( y, m, d )	( ((y-1980)<<9) + (m<<5) + d )	/* Generate file date data for the specified year, month, and day */

#define LONE_NAME_MAX_CHAR		(255*2)		/* Maximum number of characters/bytes for long file names */
#define LONG_NAME_PER_DIR		(13*2)		/* The number of characters/bytes of the long file name in each file directory information structure */

#endif

/* ********************************************************************************************************************* */
/* SCSI command and data input and output structure */

#ifndef	SPC_CMD_INQUIRY

/* SCSI command code */
#define SPC_CMD_INQUIRY			0x12
#define SPC_CMD_READ_CAPACITY	0x25
#define SPC_CMD_READ10			0x28
#define SPC_CMD_WRITE10			0x2A
#define SPC_CMD_TEST_READY		0x00
#define SPC_CMD_REQUEST_SENSE	0x03
#define SPC_CMD_MODESENSE6		0x1A
#define SPC_CMD_MODESENSE10		0x5A
#define SPC_CMD_START_STOP		0x1B

/* Command block for BulkOnly protocol */
typedef struct _BULK_ONLY_CBW {
	UINT32	CBW_Sig;
	UINT32	CBW_Tag;
	UINT8	CBW_DataLen0;					/* 08H, input: data transmission length, its valid value for input data is 0 to 48, and its valid value for output data is 0 to 33 */
	UINT8	CBW_DataLen1;
	UINT16	CBW_DataLen2;
	UINT8	CBW_Flag;						/* 0CH, input: flags such as transmission direction, if bit 7 is 1, data is input, when bit is 0, data is output or there is no data */
	UINT8	CBW_LUN;
	UINT8	CBW_CB_Len;						/* 0EH, input: the length of the command block, the valid value is 1 to 16 */
	UINT8	CBW_CB_Buf[16];					/* 0FH, input: command block, the buffer is up to 16 bytes */
} BULK_ONLY_CBW, *P_BULK_ONLY_CBW;			/* Command block of BulkOnly protocol, input CBW structure */

/* Return data of the INQUIRY command */
typedef struct _INQUIRY_DATA {
	UINT8	DeviceType;					/* 00H, device type */
	UINT8	RemovableMedia;				/* 01H, bit 7 is 1, indicating that it is mobile storage */
	UINT8	Versions;					/* 02H, protocol version */
	UINT8	DataFormatAndEtc;			/* 03H, specify the return data format */
	UINT8	AdditionalLength;			/* 04H, the length of subsequent data */
	UINT8	Reserved1;
	UINT8	Reserved2;
	UINT8	MiscFlag;					/* 07H, some control flags */
	UINT8	VendorIdStr[8];				/* 08H, manufacturer information */
	UINT8	ProductIdStr[16];			/* 10H, product information */
	UINT8	ProductRevStr[4];			/* 20H, product version */
} INQUIRY_DATA, *P_INQUIRY_DATA;		/* 24H */

/* Return data of the REQUEST SENSE command */
typedef struct _SENSE_DATA {
	UINT8	ErrorCode;					/* 00H, error code and effective bit */
	UINT8	SegmentNumber;
	UINT8	SenseKeyAndEtc;				/* 02H, primary key code */
	UINT8	Information0;
	UINT8	Information1;
	UINT8	Information2;
	UINT8	Information3;
	UINT8	AdditSenseLen;				/* 07H, the length of subsequent data */
	UINT8	CmdSpecInfo[4];
	UINT8	AdditSenseCode;				/* 0CH, additional key code */
	UINT8	AddSenCodeQual;				/* 0DH, detailed additional key code */
	UINT8	FieldReplaUnit;
	UINT8	SenseKeySpec[3];
} SENSE_DATA, *P_SENSE_DATA;			/* 12H */

#endif

/* ********************************************************************************************************************* */
/* Data input and output structures in host file mode */

#ifndef	MAX_FILE_NAME_LEN

#define MAX_FILE_NAME_LEN		(13+1)		/* The maximum length of the file name, the maximum length is 1 root directory character + 8 main file names + 1 decimal point + 3 type names + terminator = 14 */

/* Command input data and output data */
typedef union _CH376_CMD_DATA {
	struct {
		UINT8	mBuffer[ MAX_FILE_NAME_LEN ];
	} Default;

	INQUIRY_DATA	DiskMountInq;			/* Return: the return data of the INQUIRY command */
											/* CMD0H_DISK_MOUNT: Initialize the disk and test whether the disk is ready, the first time it is executed */

	FAT_DIR_INFO	OpenDirInfo;			/* Returns: the enumerated file directory information */
											/* CMD0H_FILE_OPEN: Enumerate files and directories (folders) */

	FAT_DIR_INFO	EnumDirInfo;			/* Returns: the enumerated file directory information */
											/* CMD0H_FILE_ENUM_GO: Continue to enumerate files and directories (folders) */

	struct {
		UINT8	mUpdateFileSz;				/* Input parameters: whether to allow updating the file length, 0 to prohibit updating the length */
	} FileCLose;							/* CMD1H_FILE_CLOSE: 关Close the currently opened file */

	struct {
		UINT8	mDirInfoIndex;				/* Input parameters: specify the index number of the directory information structure to be read in the sector, 0FFH is the currently opened file */
	} DirInfoRead;							/* CMD1H_DIR_INFO_READ: Read the directory information of the file */

	union {
		UINT32	mByteOffset;				/* Input parameters: number of offset bytes, offset in bytes (total length 32 bits, low byte first) */
		UINT32	mSectorLba;					/* Return: The absolute linear sector number corresponding to the current file pointer, 0FFFFFFFFH means the end of the file has been reached (total length 32 bits, low byte first) */
	} ByteLocate;							/* CMD4H_BYTE_LOCATE: move the current file pointer in bytes */

	struct {
		UINT16	mByteCount;					/* Input parameters: the number of bytes requested to be read (total length 16 bits, low byte first)*/
	} ByteRead;								/* CMD2H_BYTE_READ: Read the data block from the current position in bytes */

	struct {
		UINT16	mByteCount;					/* Input parameters: the number of bytes requested to be written (total length 16 bits, low byte first) */
	} ByteWrite;							/* CMD2H_BYTE_WRITE: Write a data block to the current location in bytes*/

	union {
		UINT32	mSectorOffset;				/* Input parameters: offset sector number, offset in sector (total length 32 bits, low byte first) */
		UINT32	mSectorLba;					/* Return: The absolute linear sector number corresponding to the current file pointer, 0FFFFFFFFH means the end of the file has been reached (total length 32 bits, low byte first) */
	} SectorLocate;							/* CMD4H_SEC_LOCATE: Move the current file pointer in units of sectors */

	struct {
		UINT8	mSectorCount;				/* Input parameter: number of sectors requested to read */
											/* Returns: the number of sectors allowed to be read */
		UINT8	mReserved1;
		UINT8	mReserved2;
		UINT8	mReserved3;
		UINT32	mStartSector;				/* Return: The starting absolute linear sector number of the sector block that is allowed to be read (total length 32 bits, low byte first) */
	} SectorRead;							/* CMD1H_SEC_READ: Read the data block from the current position in units of sectors */

	struct {
		UINT8	mSectorCount;				/* Input parameters: the number of sectors requested to be written */
											/* Returns: the number of sectors allowed to be written */
		UINT8	mReserved1;
		UINT8	mReserved2;
		UINT8	mReserved3;
		UINT32	mStartSector;				/* Return: The starting absolute linear sector number of the sector block that is allowed to be written (the total length is 32 bits, the low byte first) */
	} SectorWrite;							/* CMD1H_SEC_WRITE: Write a data block at the current position in units of sectors */

	struct {
		UINT32	mDiskSizeSec;				/* Return: the total number of sectors of the entire physical disk (the total length is 32 bits, low byte first) */
	} DiskCapacity;							/* CMD0H_DISK_CAPACITY: query disk physical capacity */

	struct {
		UINT32	mTotalSector;				/* Return: the total number of sectors of the current logical disk (total length 32 bits, low byte first) */
		UINT32	mFreeSector;				/* Return: the number of remaining sectors of the current logical disk (total length 32 bits, low byte first) */
		UINT8	mDiskFat;					/* Return: FAT type of the current logical disk, 1-FAT12, 2-FAT16, 3-FAT32 */
	} DiskQuery;							/* CMD_DiskQuery, query disk information */

	BULK_ONLY_CBW	DiskBocCbw;				/* Input parameters: CBW command structure */
											/* CMD0H_DISK_BOC_CMD: The command to execute the BulkOnly transfer protocol on the USB memory */

	struct {
		UINT8	mMaxLogicUnit;				/* Returns: the maximum logical unit number of the USB storage */
	} DiskMaxLun;							/* CMD0H_DISK_MAX_LUN: control transfer - get the maximum logical unit number of USB storage */

	INQUIRY_DATA	DiskInitInq;			/* Return: the return data of the INQUIRY command */
											/* CMD0H_DISK_INIT: Initialize USB storage */

	INQUIRY_DATA	DiskInqData;			/* Return: the return data of the INQUIRY command */
											/* CMD0H_DISK_INQUIRY: Query USB storage characteristics */

	SENSE_DATA		ReqSenseData;			/* Return: the return data of the REQUEST SENSE command */
											/* CMD0H_DISK_R_SENSE: Check for USB storage errors */

	struct {
		UINT32	mDiskSizeSec;				/* Return: the total number of sectors of the entire physical disk (total length 32 bits, high byte first) */
	} DiskSize;								/* CMD0H_DISK_SIZE: Get the size of the USB storage */

	struct {
		UINT32	mStartSector;				/* Input parameters: LBA sector address (total length 32 bits, low byte first) */
		UINT8	mSectorCount;				/* Input parameter: number of sectors requested to read */
	} DiskRead;								/* CMD5H_DISK_READ: Read data block from USB memory (in sectors) */

	struct {
		UINT32	mStartSector;				/* Input parameters: LBA sector address (total length 32 bits, low byte first) */
		UINT8	mSectorCount;				/* Input parameters: the number of sectors requested to be written */
	} DiskWrite;							/* CMD5H_DISK_WRITE: Write data block to USB memory (in sectors) */
} CH376_CMD_DATA, *P_CH376_CMD_DATA;

#endif

/* ********************************************************************************************************************* */
/* Address of a file system variable in host file mode */

#ifndef	VAR_FILE_SIZE

/* 8-bit/single-byte variable */
#define	VAR_SYS_BASE_INFO	0x20			/* Basic information about the current system */
/*           Bit 6 is used to indicate the subclass SubClass-Code of the USB storage device. If bit 6 is 0, it indicates that the subclass is 6. When bit 6 is 1, it indicates that the subclass is other than 6. */
/*           Bit 5 is used to indicate the USB configuration status in USB device mode and the USB device connection status in USB host mode */
/*                In the USB device mode, if the bit 5 is 1, the USB configuration is completed, and if the bit 5 is 0, it has not yet been configured */
/*                In USB host mode, if bit 5 is 1, there is a USB device in the USB port, and if bit 5 is 0, there is no USB device in the USB port */
/*           Bit 4 is used to indicate the buffer lock status in the USB device mode, bit 4 is 1, indicating that the USB buffer is locked, and bit 6 is 1, indicating that it has been released */
/*           Other bits, reserved, do not modify */
#define	VAR_RETRY_TIMES		0x25			/* The number of retries of USB transaction operations */
/*           If bit 7 is 0, no retry is received when NAK is received; when bit 7 is 1 and bit 6 is 0, infinite retry is received when NAK is received (you can use the CMD_ABORT_NAK command to give up retrying); when bit 7 is 1, bit 6 is 1 and received Retry up to 3 seconds when NAK */
/*           Bit 5~bit 0 is the number of retries after timeout */
#define	VAR_FILE_BIT_FLAG	0x26			/* Bit flags in host file mode */
/*           Bit 1 and bit 0, the FAT file system flag of the logical disk, 00-FAT12, 01-FAT16, 10-FAT32, 11-illegal */
/*           Bit 2, whether the FAT table data in the current buffer has been modified or not, 0-unmodified, 1-modified */
/*           Bit 3, the file length needs to be modified flag, the current file is appended with data, 0-not appended without modification, 1-appended and needs to be modified */
/*           Other bits, reserved, do not modify */
#define	VAR_DISK_STATUS		0x2B			/* Disk and file status in host file mode */
#define	VAR_SD_BIT_FLAG		0x30			/* Bit flags for SD card in host file mode */
/*           Bit 0, SD card version, 0-only support SD version 1, 1-support SD version 2 */
/*           Bit 1, automatic identification, 0-SD card, 1-MMC card */
/*           Bit 2, automatic identification, 0-standard capacity SD card, 1-high capacity SD card (HC-SD) */
/*           Bit 4, ACMD41 command timeout */
/*           Bit 5, CMD1 command timeout */
/*           Bit 6, CMD58 command timeout */
/*           Other bits, reserved, do not modify */
#define	VAR_UDISK_TOGGLE	0x31			/* Synchronization flag for the BULK-IN/BULK-OUT endpoint of the USB storage device */
/*           Bit 7, the synchronization flag of the Bulk-In endpoint */
/*           Bit 6, the synchronization flag of the Bulk-In endpoint */
/*           Bit 5~bit 0, must be 0 */
#define	VAR_UDISK_LUN		0x34			/* The logical unit number of the USB storage device */
/*           Bit 7~bit 4, the current logical unit number of the USB storage device, after the CH376 initializes the USB storage device, the default is to access the 0# logical unit */
/*           Bit 3~bit 0, the maximum logical unit number of the USB storage device, after adding 1, it is equal to the logical unit number */
#define	VAR_SEC_PER_CLUS	0x38			/* The number of sectors per cluster of the logical disk */
#define	VAR_FILE_DIR_INDEX	0x3B			/* The index number of the current file directory information in the sector */
#define	VAR_CLUS_SEC_OFS	0x3C			/* The sector offset of the current file pointer in the cluster, if it is 0xFF, it points to the end of the file, and the cluster ends */

/* 32-bit/4-byte variable */
#define	VAR_DISK_ROOT		0x44			/* For a FAT16 disk, it is the number of sectors occupied by the root directory, and for a FAT32 disk, it is the starting cluster number of the root directory (the total length is 32 bits, with the low byte first) */
#define	VAR_DSK_TOTAL_CLUS	0x48			/* The total number of clusters of the logical disk (total length 32 bits, low byte first) */
#define	VAR_DSK_START_LBA	0x4C			/* The starting absolute sector number LBA of the logical disk (total length 32 bits, low byte first) */
#define	VAR_DSK_DAT_START	0x50			/* The starting LBA of the data area of the logical disk (total length 32 bits, low byte first) */
#define	VAR_LBA_BUFFER		0x54			/* The LBA corresponding to the data in the current disk data buffer (total length 32 bits, low byte first) */
#define	VAR_LBA_CURRENT		0x58			/* The starting LBA address of the current read-write disk (total length 32 bits, low byte first) */
#define	VAR_FAT_DIR_LBA		0x5C			/* The LBA address of the sector where the current file directory information is located (total length 32 bits, low byte first) */
#define	VAR_START_CLUSTER	0x60			/* The starting cluster number of the current file or directory (folder) (total length 32 bits, low byte first) */
#define	VAR_CURRENT_CLUST	0x64			/* The current cluster number of the current file (total length 32 bits, low byte first) */
#define	VAR_FILE_SIZE		0x68			/* The length of the current file (total length 32 bits, low byte first) */
#define	VAR_CURRENT_OFFSET	0x6C			/* The current file pointer, the byte offset of the current read/write position (total length 32 bits, low byte first) */

#endif

/* ********************************************************************************************************************* */
/* Commonly used USB definitions */

/* USB packet identifier PID, which may be used in the host mode */
#ifndef	DEF_USB_PID_SETUP
#define	DEF_USB_PID_NULL	0x00			/* Reserved PID, undefined */
#define	DEF_USB_PID_SOF		0x05
#define	DEF_USB_PID_SETUP	0x0D
#define	DEF_USB_PID_IN		0x09
#define	DEF_USB_PID_OUT		0x01
#define	DEF_USB_PID_ACK		0x02
#define	DEF_USB_PID_NAK		0x0A
#define	DEF_USB_PID_STALL	0x0E
#define	DEF_USB_PID_DATA0	0x03
#define	DEF_USB_PID_DATA1	0x0B
#define	DEF_USB_PID_PRE		0x0C
#endif

/* USB request type, may be used in external firmware mode */
#ifndef	DEF_USB_REQ_TYPE
#define	DEF_USB_REQ_READ	0x80			/* Control read operation */
#define	DEF_USB_REQ_WRITE	0x00			/* Control write operation */
#define	DEF_USB_REQ_TYPE	0x60			/* Control request type */
#define	DEF_USB_REQ_STAND	0x00			/* Standard request */
#define	DEF_USB_REQ_CLASS	0x20			/* Device class request */
#define	DEF_USB_REQ_VENDOR	0x40			/* Vendor request */
#define	DEF_USB_REQ_RESERVE	0x60			/* Reserve request */
#endif

/* USB standard device request, bit 6 bit 5=00(Standard) of RequestType, may be used in external firmware mode */
#ifndef	DEF_USB_GET_DESCR
#define	DEF_USB_CLR_FEATURE	0x01
#define	DEF_USB_SET_FEATURE	0x03
#define	DEF_USB_GET_STATUS	0x00
#define	DEF_USB_SET_ADDRESS	0x05
#define	DEF_USB_GET_DESCR	0x06
#define	DEF_USB_SET_DESCR	0x07
#define	DEF_USB_GET_CONFIG	0x08
#define	DEF_USB_SET_CONFIG	0x09
#define	DEF_USB_GET_INTERF	0x0A
#define	DEF_USB_SET_INTERF	0x0B
#define	DEF_USB_SYNC_FRAME	0x0C
#endif

/* ********************************************************************************************************************* */

#ifdef __cplusplus
}
#endif

#endif
