/*
 * Copyright (C) 2020 - 2022 SG Wireless Limited - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 *
 */
/**
 * @file ch376_fs.c
 * @brief
 */
/***********************************************Include***********************************************/
#include <nrf_delay.h>
#include <stdio.h>

/* FreeRTOS */
#include "FreeRTOS.h"
#include "task.h"
/***********************************Global Definitions and Declarations*******************************/
/***********************************Constant / Macro Definitions**************************************/
/*******************************************Type Declarations*****************************************/
/******************************************Function Declarations**************************************/
/******************************************Variable Declarations**************************************/
/*******************************************Function Prototype****************************************/
/* CH376 chip file system layer V1.3 */
/* Provide common subroutines of the file system, and provide command packaging */
/* Subroutines that are not used can be commented out, thereby saving the program ROM space and data RAM space of the microcontroller */
/* The subroutines here pass parameters through the variables in brackets. If there are many parameters, in order to save RAM, you can also refer to the CH375 subroutine library and change them to pass through the same global variable/joint structure CH376_CMD_DATA
 */

/* The name parameter refers to the short file name, which can include the root directory character, but does not contain the path separator, and the total length does not exceed 1+8+1+3+1 bytes */
/* The PathName parameter refers to the short file name of the full path, including the root directory character, multi-level subdirectories and path separators, file name/directory name */
/* The LongName parameter refers to the long file name, which is encoded in UNICODE little endian order and ends with two 0 bytes. To use the long file name subroutine, the global buffer GlobalBuf must first be defined with a length of not less than 64 bytes, which can be shared with other subroutines */

/* Define NO_DEFAULT_CH376_INT to disable the default Wait376Interrupt subroutine, after the prohibition, the application must define a subroutine with the same name */
/* The definition DEF_INT_TIMEOUT is used to set the timeout time/cycle count value of the waiting interrupt in the default Wait376Interrupt subroutine, and 0 does not check the timeout and waits all the time */
/* Define EN_DIR_CREATE to provide a subroutine for creating a new multi-level subdirectory, which is not provided by default */
/* Define EN_DISK_QUERY to provide subroutines for disk capacity query and remaining space query, the default is not provided */
/* Define EN_SECTOR_ACCESS to provide subroutines for reading and writing files in units of sectors, the default is not provided */
/* Define EN_LONG_NAME to provide subroutines that support long file names, the default is not provided */
/* Define DEF_IC_V43_U to remove the program codes that support lower versions, only support CH376 chips of V4.3 and above, and support lower versions by default */

#if 1
#define DEF_IC_V43_U 1 /* It is recommended to define DEF_IC_V43_U to optimize the code */
#endif

#define DEF_INT_TIMEOUT 1000

#include "ch376_fs.h"
extern void xWriteCH376Cmd(uint8_t cmd);
extern uint8_t xReadCH376Data(void);
extern void xWriteCH376Data(UINT8 data);
extern void xEndCH376Cmd(void);
extern bool Query376Interrupt(void);

UINT8 CH376ReadBlock(PUINT8 buf) /* Read a block of data from the receive buffer of the current host endpoint and return the length */
{
    UINT8 s, l;
    xWriteCH376Cmd(CMD01_RD_USB_DATA0);
    s = l = xReadCH376Data(); /* Length */
    if (l) {
        do {
            *buf = xReadCH376Data();
            buf++;
        } while (--l);
    }
    xEndCH376Cmd();
    return (s);
}

UINT8 CH376WriteReqBlock(PUINT8 buf) /* Write the requested data block to the internal designated buffer and return the length */
{
    UINT8 s, l;
    xWriteCH376Cmd(CMD01_WR_REQ_DATA);
    s = l = xReadCH376Data(); /* Length */
    if (l) {
        do {
            xWriteCH376Data(*buf);
            buf++;
        } while (--l);
    }
    xEndCH376Cmd();
    return (s);
}

void CH376WriteHostBlock(PUINT8 buf, UINT8 len) /* Write a block of data to the send buffer of the USB host endpoint */
{
    xWriteCH376Cmd(CMD10_WR_HOST_DATA);
    xWriteCH376Data(len); /* Length  */
    if (len) {
        do {
            xWriteCH376Data(*buf);
            buf++;
        } while (--len);
    }
    xEndCH376Cmd();
}

void CH376WriteOfsBlock(PUINT8 buf, UINT8 ofs, UINT8 len) /* Write a data block to the specified offset address of the internal buffer */
{
    xWriteCH376Cmd(CMD20_WR_OFS_DATA);
    xWriteCH376Data(ofs); /* Offset address */
    xWriteCH376Data(len); /* Length  */
    if (len) {
        do {
            xWriteCH376Data(*buf);
            buf++;
        } while (--len);
    }
    xEndCH376Cmd();
}

void CH376SetFileName(PUINT8 name) /* Set the filename of the file to be operated on */
{
    /*	UINT8	i;*/
    UINT8 c;
#ifndef DEF_IC_V43_U
    UINT8 s;
    xWriteCH376Cmd(CMD01_GET_IC_VER);
    if (xReadCH376Data() < 0x43) {
        if (CH376ReadVar8(VAR_DISK_STATUS) < DEF_DISK_READY) {
            xWriteCH376Cmd(CMD10_SET_FILE_NAME);
            xWriteCH376Data(0);
            s = CH376SendCmdWaitInt(CMD0H_FILE_OPEN);
            if (s == USB_INT_SUCCESS) {
                s = CH376ReadVar8(0xCF);
                if (s) {
                    CH376WriteVar32(0x4C, CH376ReadVar32(0x4C) + ((UINT16)s << 8));
                    CH376WriteVar32(0x50, CH376ReadVar32(0x50) + ((UINT16)s << 8));
                    CH376WriteVar32(0x70, 0);
                }
            }
        }
    }
    xEndCH376Cmd();
#endif
    xWriteCH376Cmd(CMD10_SET_FILE_NAME);
    /*	for ( i = MAX_FILE_NAME_LEN; i != 0; -- i ) {
                    c = *name;
                    xWriteCH376Data( c );
                    if ( c == 0 ) break;
                    name ++;
            }*/
    c = *name;
    xWriteCH376Data(c);
    while (c) {
        name++;
        c = *name;
        if (c == DEF_SEPAR_CHAR1 || c == DEF_SEPAR_CHAR2)
            c = 0; /* Forcibly terminating filenames */
        xWriteCH376Data(c);
    }
    xEndCH376Cmd();
}

UINT32 CH376Read32bitDat(void) /* Read 32-bit data from CH376 chip and end the command */
{
    UINT8 c0, c1, c2, c3;
    c0 = xReadCH376Data();
    c1 = xReadCH376Data();
    c2 = xReadCH376Data();
    c3 = xReadCH376Data();
    xEndCH376Cmd();
    return (c0 | (UINT16)c1 << 8 | (UINT32)c2 << 16 | (UINT32)c3 << 24);
}

UINT8 CH376ReadVar8(UINT8 var) /* Read the 8-bit variable inside the CH376 chip */
{
    UINT8 c0;
    xWriteCH376Cmd(CMD11_READ_VAR8);
    xWriteCH376Data(var);
    c0 = xReadCH376Data();
    xEndCH376Cmd();
    return (c0);
}

void CH376WriteVar8(UINT8 var, UINT8 dat) /* Write the 8-bit variable inside the CH376 chip */
{
    xWriteCH376Cmd(CMD20_WRITE_VAR8);
    xWriteCH376Data(var);
    xWriteCH376Data(dat);
    xEndCH376Cmd();
}

UINT32 CH376ReadVar32(UINT8 var) /* Read the 32-bit variables inside the CH376 chip */
{
    xWriteCH376Cmd(CMD14_READ_VAR32);
    xWriteCH376Data(var);
    return (CH376Read32bitDat()); /* Read 32-bit data from CH376 chip and end the command */
}

void CH376WriteVar32(UINT8 var, UINT32 dat) /* Write the 32-bit variables inside the CH376 chip */
{
    xWriteCH376Cmd(CMD50_WRITE_VAR32);
    xWriteCH376Data(var);
    xWriteCH376Data((UINT8)dat);
    xWriteCH376Data((UINT8)((UINT16)dat >> 8));
    xWriteCH376Data((UINT8)(dat >> 16));
    xWriteCH376Data((UINT8)(dat >> 24));
    xEndCH376Cmd();
}

void CH376EndDirInfo(void) /* After calling CH376DirInfoRead to get the FAT_DIR_INFO structure, CH376 should be notified to end */
{
    CH376WriteVar8(0x0D, 0x00);
}

UINT32 CH376GetFileSize(void) /* Read current file length */
{
    return (CH376ReadVar32(VAR_FILE_SIZE));
}

UINT8 CH376GetDiskStatus(void) /* Get the working status of disks and filesystems */
{
    return (CH376ReadVar8(VAR_DISK_STATUS));
}

UINT8 CH376GetIntStatus(void) /* Get interrupt status and cancel interrupt request */
{
    UINT8 s;
    xWriteCH376Cmd(CMD01_GET_STATUS);
    s = xReadCH376Data();
    xEndCH376Cmd();
    return (s);
}

#ifndef NO_DEFAULT_CH376_INT
UINT8 Wait376Interrupt(void) /* Wait for CH376 interrupt (INT# low level), return interrupt status code, return ERR_USB_UNKNOWN when timeout */
{
#ifdef DEF_INT_TIMEOUT
#if DEF_INT_TIMEOUT < 1
    while (Query376Interrupt() == FALSE)
        ;                         /* Wait for interruption */
    return (CH376GetIntStatus()); /* Interrupt detected */
#else
    UINT32 i;
    for (i = 0; i < DEF_INT_TIMEOUT; i++) { /* Count prevents timeout */
        if (Query376Interrupt())
            return (CH376GetIntStatus()); /*Interrupt detected */
        /* In the process of waiting for the CH376 interrupt, you can do other things that need to be processed in time */
        vTaskDelay(1);
    }
    return (-1); /* What shouldn't happen */
#endif
#else
    UINT32 i;
    for (i = 0; i < 500000; i++) { /* Counting prevents timeout, the default timeout time is related to the main frequency of the microcontroller */
        if (Query376Interrupt())
            return (CH376GetIntStatus()); /* Interrupt detected */
        else
            nrf_delay_us(100);
        /* In the process of waiting for the CH376 interrupt, you can do other things that need to be processed in time */
    }
    return (-1); /* What shouldn't happen */
#endif
}
#endif

UINT8 CH376SendCmdWaitInt(UINT8 mCmd) /* After sending the command code, wait for the interrupt */
{
    xWriteCH376Cmd(mCmd);
    xEndCH376Cmd();
    return (Wait376Interrupt());
}

UINT8 CH376SendCmdDatWaitInt(UINT8 mCmd, UINT8 mDat) /* After sending the command code and one byte of data, wait for the interrupt */
{
    xWriteCH376Cmd(mCmd);
    xWriteCH376Data(mDat);
    xEndCH376Cmd();
    return (Wait376Interrupt());
}

UINT8 CH376DiskReqSense(void) /* Check USB memory for errors */
{
    UINT8 s;
    vTaskDelay(5);
    s = CH376SendCmdWaitInt(CMD0H_DISK_R_SENSE);
    vTaskDelay(5);
    return (s);
}

UINT8 CH376DiskConnect(void) /* Check whether the U disk is connected, SD card is not supported */
{
    if (Query376Interrupt()) {
        CH376GetIntStatus(); /* Interrupt detected */
    }
    return (CH376SendCmdWaitInt(CMD0H_DISK_CONNECT));
}

UINT8 CH376DiskMount(void) /* Initialize the disk and test whether the disk is ready */
{
    if (Query376Interrupt())
        CH376GetIntStatus();
    return (CH376SendCmdWaitInt(CMD0H_DISK_MOUNT));
}

UINT8 CH376FileOpen(PUINT8 name) /* Open a file or directory (folder) in the root directory or the current directory */
{
    CH376SetFileName(name); /* Set the filename of the file to be operated on */
#ifndef DEF_IC_V43_U
    if (name[0] == DEF_SEPAR_CHAR1 || name[0] == DEF_SEPAR_CHAR2)
        CH376WriteVar32(VAR_CURRENT_CLUST, 0);
#endif

    return (CH376SendCmdWaitInt(CMD0H_FILE_OPEN));
}

UINT8 CH376FileCreate(PUINT8 name) /* Create a new file in the root directory or the current directory, if the file already exists, delete it first */
{
    if (name)
        CH376SetFileName(name); /* 设Set the filename of the file to be operated on */
    return (CH376SendCmdWaitInt(CMD0H_FILE_CREATE));
}

UINT8 CH376DirCreate(PUINT8 name) /* Create a new directory (folder) under the root directory and open it, if the directory already exists, open it directly */
{
    CH376SetFileName(name); /* Set the filename of the file to be operated on */
#ifndef DEF_IC_V43_U
    if (name[0] == DEF_SEPAR_CHAR1 || name[0] == DEF_SEPAR_CHAR2)
        CH376WriteVar32(VAR_CURRENT_CLUST, 0);
#endif
    return (CH376SendCmdWaitInt(CMD0H_DIR_CREATE));
}

UINT8 CH376SeparatePath(
    PUINT8 path) /* eparate the last-level file name or directory (folder) name from the path, and return the byte offset of the last-level file name or directory name */
{
    PUINT8 pName;
    for (pName = path; *pName != 0; ++pName)
        ; /* To the end of the filename string */
    while (*pName != DEF_SEPAR_CHAR1 && *pName != DEF_SEPAR_CHAR2 && pName != path)
        pName--; /*  Search for the last but one path separator */
    if (pName != path)
        pName++; /* If the path separator is found, modify the last level file name pointing to the target file, skip the previous multi-level directory name and path separator */
    return (pName - path);
}

UINT8 CH376FileOpenDir(PUINT8 PathName,
    UINT8 StopName) /* Open a file in a multi-level directory or the parent directory of a directory, support multi-level directory paths, support path separators, and the path length does not exceed 255 characters */
/* StopName points to the last level file name or directory name*/
{
    UINT8 i, s;
    s = 0;
    i = 1; /* Skip possible root directory identifiers */
    while (1) {
        while (PathName[i] != DEF_SEPAR_CHAR1 && PathName[i] != DEF_SEPAR_CHAR2 && PathName[i] != 0)
            ++i; /* Search for the next path separator or path terminator */
        if (PathName[i])
            i++; /* The path separator is found, modify the last level file name pointing to the target file */
        else
            i = 0;                       /* End of path */
        s = CH376FileOpen(&PathName[s]); /* Open file or directory */
        if (i && i != StopName) {        /* The path is not over */
            if (s != ERR_OPEN_DIR) { /* Because it is opened step by step, the path has not yet reached the end, so if the directory is not successfully opened, then there is a problem */
                if (s == USB_INT_SUCCESS)
                    return (ERR_FOUND_NAME); /* The intermediate path must be a directory name, if it is a file name, an error will occur */
                else if (s == ERR_MISS_FILE)
                    return (ERR_MISS_DIR); /* A subdirectory in the middle path was not found, maybe the directory name is wrong */
                else
                    return (s); /* Operation error */
            }
            s = i; /* Continue from the next directory */
        } else
            return (s); /* When the path ends, USB_INT_SUCCESS means that the file was successfully opened, ERR_OPEN_DIR means that the directory (folder) was successfully opened, and other means that an operation error occurred */
    }
}

UINT8 CH376FileOpenPath(
    PUINT8 PathName) /* Open files or directories (folders) under multi-level directories, support multi-level directory paths, support path separators, and the path length does not exceed 255 characters */
{
    return (CH376FileOpenDir(PathName, 0xFF));
}

UINT8 CH376FileCreatePath(
    PUINT8 PathName) /* Create a file under a multi-level directory, support multi-level directory paths, support path separators, and the path length does not exceed 255 characters */
{
    UINT8 s;
    UINT8 Name;
    Name = CH376SeparatePath(PathName); /* Separate the last-level filename from the path, and return the offset of the last-level filename */
    if (Name) {                         /* Is a multi-level directory */
        s = CH376FileOpenDir(PathName, Name); /* Open the last-level directory under the multi-level directory, that is, open the upper-level directory of the newly created file */
        if (s != ERR_OPEN_DIR) { /* Because the upper-level directory is opened, if the directory is not successfully opened, then there is a problem */
            if (s == USB_INT_SUCCESS)
                return (ERR_FOUND_NAME); /* The intermediate path must be a directory name, if it is a file name, an error will occur */
            else if (s == ERR_MISS_FILE)
                return (ERR_MISS_DIR); /* A subdirectory in the middle path was not found, maybe the directory name is wrong */
            else
                return (s); /* Operation error */
        }
    }
    return (CH376FileCreate(&PathName[Name])); /* Create a new file in the root directory or the current directory */
}

#ifdef EN_DIR_CREATE
UINT8 CH376DirCreatePath(
    PUINT8 PathName) /* Create a new directory (folder) under the multi-level directory and open it, support multi-level directory paths, support path separators, and the path length does not exceed 255 characters */
{
    UINT8 s;
    UINT8 Name;
    UINT8 ClustBuf[4];
    Name = CH376SeparatePath(PathName); /* Separate the last-level directory name from the path, and return the offset of the last-level file name */
    if (Name) {                         /* Is a multi-level directory */
        s = CH376FileOpenDir(PathName, Name); /* Open the last-level directory under the multi-level directory, that is, open the upper-level directory of the newly created directory */
        if (s != ERR_OPEN_DIR) { /* Because the upper-level directory is opened, if the directory is not successfully opened, then there is a problem*/
            if (s == USB_INT_SUCCESS)
                return (ERR_FOUND_NAME); /* The intermediate path must be a directory name, if it is a file name, an error will occur */
            else if (s == ERR_MISS_FILE)
                return (ERR_MISS_DIR); /* A subdirectory in the middle path was not found, maybe the directory name is wrong */
            else
                return (s); /* Operation error */
        }
        xWriteCH376Cmd(CMD14_READ_VAR32);
        xWriteCH376Data(VAR_START_CLUSTER); /* The starting cluster number of the parent directory */
        for (s = 0; s != 4; s++)
            ClustBuf[s] = xReadCH376Data();
        xEndCH376Cmd();
        s = CH376DirCreate(&PathName[Name]); /* Create a new directory under the current directory */
        if (s != USB_INT_SUCCESS)
            return (s);
        CH376WriteVar32(VAR_FILE_SIZE, sizeof(FAT_DIR_INFO) * 2);
        s = CH376ByteLocate(sizeof(FAT_DIR_INFO) + STRUCT_OFFSET(FAT_DIR_INFO, DIR_FstClusHI)); /* Move file pointer */
        if (s != USB_INT_SUCCESS)
            return (s);
        s = CH376ByteWrite(&ClustBuf[2], 2, NULL); /* The upper 16 bits of the starting cluster number written to the upper-level directory */
        if (s != USB_INT_SUCCESS)
            return (s);
        s = CH376ByteLocate(sizeof(FAT_DIR_INFO) + STRUCT_OFFSET(FAT_DIR_INFO, DIR_FstClusLO)); /* Move file pointer */
        if (s != USB_INT_SUCCESS)
            return (s);
        s = CH376ByteWrite(ClustBuf, 2, NULL); /* The lower 16 bits of the starting cluster number written to the upper-level directory */
        if (s != USB_INT_SUCCESS)
            return (s);
        s = CH376ByteLocate(0); /* Move the file pointer and restore to the directory header position */
        if (s != USB_INT_SUCCESS)
            return (s);
        CH376WriteVar32(VAR_FILE_SIZE, 0);
        return (s);
    } else { /* Not a multi-level directory */
        if (PathName[0] == DEF_SEPAR_CHAR1 || PathName[0] == DEF_SEPAR_CHAR2)
            return (CH376DirCreate(PathName)); /* Create a new directory under the root directory */
        else
            return (ERR_MISS_DIR); /* A full path must be provided to create a new directory under the current directory */
    }
}
#endif

UINT8 CH376FileErase(PUINT8 PathName) /* Delete the file, if it has been opened, delete it directly, otherwise the file will be opened first and then deleted, support multi-level directory path */
{
    UINT8 s;
    if (PathName) { /* File has not been opened */
        for (s = 1; PathName[s] != DEF_SEPAR_CHAR1 && PathName[s] != DEF_SEPAR_CHAR2 && PathName[s] != 0; ++s)
            ;                                /* Search for the next path separator or path terminator */
        if (PathName[s]) {                   /* With path separators, it is a file or directory under a multi-level directory */
            s = CH376FileOpenPath(PathName); /* Open a file or directory under a multi-level directory */
            if (s != USB_INT_SUCCESS && s != ERR_OPEN_DIR)
                return (s); /* Operation error*/
        } else
            CH376SetFileName(
                PathName); /* There is no path separator, it is a file or directory under the root directory or the current directory, set the file name of the file to be operated */
    }
    return (CH376SendCmdWaitInt(CMD0H_FILE_ERASE));
}

UINT8 CH376FileClose(UINT8 UpdateSz) /* Close the currently opened file or directory (folder) */
{
    return (CH376SendCmdDatWaitInt(CMD1H_FILE_CLOSE, UpdateSz));
}

UINT8 CH376DirInfoRead(void) /* Read the directory information of the current file */
{
    return (CH376SendCmdDatWaitInt(CMD1H_DIR_INFO_READ, 0xFF));
}

UINT8 CH376DirInfoSave(void) /* Save the directory information of the file */
{
    return (CH376SendCmdWaitInt(CMD0H_DIR_INFO_SAVE));
}

UINT8 CH376ByteLocate(UINT32 offset) /* Move the current file pointer in bytes */
{
    xWriteCH376Cmd(CMD4H_BYTE_LOCATE);
    xWriteCH376Data((UINT8)offset);
    xWriteCH376Data((UINT8)((UINT16)offset >> 8));
    xWriteCH376Data((UINT8)(offset >> 16));
    xWriteCH376Data((UINT8)(offset >> 24));
    xEndCH376Cmd();
    return (Wait376Interrupt());
}

UINT8 CH376ByteRead(PUINT8 buf, UINT16 ReqCount, PUINT16 RealCount) /* Read a block of data from the current position in bytes */
{
    UINT8 s;
    xWriteCH376Cmd(CMD2H_BYTE_READ);
    xWriteCH376Data((UINT8)ReqCount);
    xWriteCH376Data((UINT8)(ReqCount >> 8));
    xEndCH376Cmd();
    if (RealCount)
        *RealCount = 0;
    while (1) {
        s = Wait376Interrupt();
        if (s == USB_INT_DISK_READ) {
            s = CH376ReadBlock(buf); /* Read a block of data from the receive buffer of the current host endpoint and return the length */
            xWriteCH376Cmd(CMD0H_BYTE_RD_GO);
            xEndCH376Cmd();
            buf += s;
            if (RealCount)
                *RealCount += s;
        }
        /*		else if ( s == USB_INT_SUCCESS ) return( s );*/ /* Finish */
        else
            return (s); /* 错误 */
    }
}

UINT8 CH376ByteWrite(PUINT8 buf, UINT16 ReqCount, PUINT16 RealCount) /* Write a data block to the current location in bytes */
{
    UINT8 s;
    xWriteCH376Cmd(CMD2H_BYTE_WRITE);
    xWriteCH376Data((UINT8)ReqCount);
    xWriteCH376Data((UINT8)(ReqCount >> 8));
    xEndCH376Cmd();
    if (RealCount)
        *RealCount = 0;
    while (1) {
        s = Wait376Interrupt();
        if (s == USB_INT_DISK_WRITE) {
            s = CH376WriteReqBlock(buf); /* Write the requested data block to the internal designated buffer and return the length */
            xWriteCH376Cmd(CMD0H_BYTE_WR_GO);
            xEndCH376Cmd();
            buf += s;
            if (RealCount)
                *RealCount += s;
        }
        /*		else if ( s == USB_INT_SUCCESS ) return( s );*/ /* Finish */
        else
            return (s); /* Error */
    }
}

#ifdef EN_DISK_QUERY

UINT8 CH376DiskCapacity(PUINT32 DiskCap) /* Query disk physical capacity, number of sectors */
{
    UINT8 s;
    s = CH376SendCmdWaitInt(CMD0H_DISK_CAPACITY);
    if (s == USB_INT_SUCCESS) { /* Refer to the DiskCapacity of the CH376_CMD_DATA structure in the CH376INC.H file */
        xWriteCH376Cmd(CMD01_RD_USB_DATA0);
        xReadCH376Data(); /* The length is always sizeof(CH376_CMD_DATA.DiskCapacity) */
        *DiskCap =
            CH376Read32bitDat(); /* CH376_CMD_DATA.DiskCapacity.mDiskSizeSec, read 32-bit data from CH376 chip and end the command */
    } else
        *DiskCap = 0;
    return (s);
}

UINT8 CH376DiskQuery(PUINT32 DiskFre) /* Query disk remaining space information, number of sectors */
{
    UINT8 s;
    UINT8 c0, c1, c2, c3;
#ifndef DEF_IC_V43_U
    xWriteCH376Cmd(CMD01_GET_IC_VER);
    if (xReadCH376Data() < 0x43) {
        if (CH376ReadVar8(VAR_DISK_STATUS) >= DEF_DISK_READY)
            CH376WriteVar8(VAR_DISK_STATUS, DEF_DISK_MOUNTED);
    }
#endif
    s = CH376SendCmdWaitInt(CMD0H_DISK_QUERY);
    if (s == USB_INT_SUCCESS) { /* Refer to the DiskQuery of the CH376_CMD_DATA structure in the CH376INC.H file */
        xWriteCH376Cmd(CMD01_RD_USB_DATA0);
        xReadCH376Data(); /* The length is always sizeof(CH376_CMD_DATA.DiskQuery) */
        xReadCH376Data(); /* CH376_CMD_DATA.DiskQuery.mTotalSector */
        xReadCH376Data();
        xReadCH376Data();
        xReadCH376Data();
        c0       = xReadCH376Data(); /* CH376_CMD_DATA.DiskQuery.mFreeSector */
        c1       = xReadCH376Data();
        c2       = xReadCH376Data();
        c3       = xReadCH376Data();
        *DiskFre = c0 | (UINT16)c1 << 8 | (UINT32)c2 << 16 | (UINT32)c3 << 24;
        xReadCH376Data(); /* CH376_CMD_DATA.DiskQuery.mDiskFat */
        xEndCH376Cmd();
    } else
        *DiskFre = 0;
    return (s);
}

#endif

UINT8 CH376SecLocate(UINT32 offset) /* Move the current file pointer in units of sectors */
{
    xWriteCH376Cmd(CMD4H_SEC_LOCATE);
    xWriteCH376Data((UINT8)offset);
    xWriteCH376Data((UINT8)((UINT16)offset >> 8));
    xWriteCH376Data((UINT8)(offset >> 16));
    xWriteCH376Data(0); /* Maximum file size exceeded */
    xEndCH376Cmd();
    return (Wait376Interrupt());
}

#ifdef EN_SECTOR_ACCESS

UINT8 CH376DiskReadSec(
    PUINT8 buf, UINT32 iLbaStart, UINT8 iSectorCount) /* Read data blocks of multiple sectors from the U disk to the buffer, does not support SD cards */
/* iLbaStart is the linear start sector number to be read, iSectorCount is the number of sectors to be read*/
{
    UINT8 s, err;
    UINT16 mBlockCount;
    for (err = 0; err != 3; ++err) {       /* Error retry*/
        xWriteCH376Cmd(CMD5H_DISK_READ);   /* Read sectors from USB memory */
        xWriteCH376Data((UINT8)iLbaStart); /* Lowest 8 bits of LBA */
        xWriteCH376Data((UINT8)((UINT16)iLbaStart >> 8));
        xWriteCH376Data((UINT8)(iLbaStart >> 16));
        xWriteCH376Data((UINT8)(iLbaStart >> 24)); /* Highest 8 bits of LBA */
        xWriteCH376Data(iSectorCount);             /* Number of sectors */
        xEndCH376Cmd();
        for (mBlockCount = iSectorCount * DEF_SECTOR_SIZE / CH376_DAT_BLOCK_LEN; mBlockCount != 0;
             --mBlockCount) {             /* Block count */
            s = Wait376Interrupt();       /* Wait for interrupt and get status */
            if (s == USB_INT_DISK_READ) { /* USB memory read data block, request data read */
                s = CH376ReadBlock(buf);  /* Read a block of data from the receive buffer of the current host endpoint and return the length */
                xWriteCH376Cmd(CMD0H_DISK_RD_GO); /* Continue to execute the read operation of the USB memory */
                xEndCH376Cmd();
                buf += s;
            } else
                break; /* Return error status */
        }
        if (mBlockCount == 0) {
            s = Wait376Interrupt(); /* Wait for interrupt and get status */
            if (s == USB_INT_SUCCESS)
                return (USB_INT_SUCCESS); /* Successful operation */
        }
        if (s == USB_INT_DISCONNECT)
            return (s);      /* U disk is removed */
        CH376DiskReqSense(); /* Check USB memory for errors */
    }
    return (s); /* Operation failed */
}

UINT8 CH376DiskWriteSec(
    PUINT8 buf, UINT32 iLbaStart, UINT8 iSectorCount) /* Write the data blocks of multiple sectors in the buffer to the U disk, SD card is not supported */
/* iLbaStart is the line start sector number written, iSectorCount is the number of sectors written */
{
    UINT8 s, err;
    UINT16 mBlockCount;
    for (err = 0; err != 3; ++err) {       /* Error retry*/
        xWriteCH376Cmd(CMD5H_DISK_WRITE);  /* Read sectors from USB memory */
        xWriteCH376Data((UINT8)iLbaStart); /* Lowest 8 bits of LBA  */
        xWriteCH376Data((UINT8)((UINT16)iLbaStart >> 8));
        xWriteCH376Data((UINT8)(iLbaStart >> 16));
        xWriteCH376Data((UINT8)(iLbaStart >> 24)); /* Highest 8 bits of LBA */
        xWriteCH376Data(iSectorCount);             /* Number of sectors  */
        xEndCH376Cmd();
        for (mBlockCount = iSectorCount * DEF_SECTOR_SIZE / CH376_DAT_BLOCK_LEN; mBlockCount != 0;
             --mBlockCount) {                                  /* Block count */
            s = Wait376Interrupt();                            /* Wait for interrupt and get status */
            if (s == USB_INT_DISK_WRITE) {                     /* USB memory write data block, request data write */
                CH376WriteHostBlock(buf, CH376_DAT_BLOCK_LEN); /* Write a block of data to the send buffer of the USB host endpoint */
                xWriteCH376Cmd(CMD0H_DISK_WR_GO);              /* Continue to execute the write operation of the USB memory */
                xEndCH376Cmd();
                buf += CH376_DAT_BLOCK_LEN;
            } else
                break; /* Return error status */
        }
        if (mBlockCount == 0) {
            s = Wait376Interrupt(); /* Wait for interrupt and get status */
            if (s == USB_INT_SUCCESS)
                return (USB_INT_SUCCESS); /* Successful operation */
        }
        if (s == USB_INT_DISCONNECT)
            return (s);      /* U disk is removed */
        CH376DiskReqSense(); /* Check USB memory for errors */
    }
    return (s); /* Operation failed */
}

UINT8 CH376SecRead(PUINT8 buf, UINT8 ReqCount, PUINT8 RealCount) /* Read the data block from the current position in units of sectors, SD card is not supported */
{
    UINT8 s;
    UINT8 cnt;
    UINT32 StaSec;
#ifndef DEF_IC_V43_U
    UINT32 fsz, fofs;
#endif
    if (RealCount)
        *RealCount = 0;
    do {
#ifndef DEF_IC_V43_U
        xWriteCH376Cmd(CMD01_GET_IC_VER);
        cnt = xReadCH376Data();
        if (cnt == 0x41) {
            xWriteCH376Cmd(CMD14_READ_VAR32);
            xWriteCH376Data(VAR_FILE_SIZE);
            xReadCH376Data();
            fsz = xReadCH376Data();
            fsz |= (UINT16)(xReadCH376Data()) << 8;
            cnt = xReadCH376Data();
            fsz |= (UINT32)cnt << 16;
            xWriteCH376Cmd(CMD14_READ_VAR32);
            xWriteCH376Data(VAR_CURRENT_OFFSET);
            xReadCH376Data();
            fofs = xReadCH376Data();
            fofs |= (UINT16)(xReadCH376Data()) << 8;
            fofs |= (UINT32)(xReadCH376Data()) << 16;
            if (fsz >= fofs + 510)
                CH376WriteVar8(VAR_FILE_SIZE + 3, 0xFF);
            else
                cnt = 0xFF;
        } else
            cnt = 0xFF;
#endif
        xWriteCH376Cmd(CMD1H_SEC_READ);
        xWriteCH376Data(ReqCount);
        xEndCH376Cmd();
        s = Wait376Interrupt();
#ifndef DEF_IC_V43_U
        if (cnt != 0xFF)
            CH376WriteVar8(VAR_FILE_SIZE + 3, cnt);
#endif
        if (s != USB_INT_SUCCESS)
            return (s);
        xWriteCH376Cmd(CMD01_RD_USB_DATA0);
        xReadCH376Data();       /* The length is always sizeof(CH376_CMD_DATA.SectorRead) */
        cnt = xReadCH376Data(); /* CH376_CMD_DATA.SectorRead.mSectorCount */
        xReadCH376Data();
        xReadCH376Data();
        xReadCH376Data();
        StaSec = CH376Read32bitDat(); /* CH376_CMD_DATA.SectorRead.mStartSector,read 32-bit data from CH376 chip and end the command */
        if (cnt == 0)
            break;
        s = CH376DiskReadSec(buf, StaSec, cnt); /* Read data blocks of multiple sectors from the U disk to the buffer */
        if (s != USB_INT_SUCCESS)
            return (s);
        buf += cnt * DEF_SECTOR_SIZE;
        if (RealCount)
            *RealCount += cnt;
        ReqCount -= cnt;
    } while (ReqCount);
    return (s);
}

UINT8 CH376SecWrite(PUINT8 buf, UINT8 ReqCount, PUINT8 RealCount) /* Write a data block at the current position in units of sectors, SD card is not supported */
{
    UINT8 s;
    UINT8 cnt;
    UINT32 StaSec;
    if (RealCount)
        *RealCount = 0;
    do {
        xWriteCH376Cmd(CMD1H_SEC_WRITE);
        xWriteCH376Data(ReqCount);
        xEndCH376Cmd();
        s = Wait376Interrupt();
        if (s != USB_INT_SUCCESS)
            return (s);
        xWriteCH376Cmd(CMD01_RD_USB_DATA0);
        xReadCH376Data();       /* The length is always sizeof(CH376_CMD_DATA.SectorWrite) */
        cnt = xReadCH376Data(); /* CH376_CMD_DATA.SectorWrite.mSectorCount */
        xReadCH376Data();
        xReadCH376Data();
        xReadCH376Data();
        StaSec = CH376Read32bitDat(); /* CH376_CMD_DATA.SectorWrite.mStartSector, read 32-bit data from CH376 chip and end the command */
        if (cnt == 0)
            break;
        s = CH376DiskWriteSec(buf, StaSec, cnt); /* Write the data blocks of multiple sectors in the buffer to the U disk */
        if (s != USB_INT_SUCCESS)
            return (s);
        buf += cnt * DEF_SECTOR_SIZE;
        if (RealCount)
            *RealCount += cnt;
        ReqCount -= cnt;
    } while (ReqCount);
    return (s);
}

#endif

#ifdef EN_LONG_NAME

UINT8 CH376LongNameWrite(PUINT8 buf, UINT16 ReqCount) /* Byte-writing subroutines for long filenames */
{
    UINT8 s;
#ifndef DEF_IC_V43_U
    UINT8 c;
    c = CH376ReadVar8(VAR_DISK_STATUS);
    if (c == DEF_DISK_OPEN_ROOT)
        CH376WriteVar8(VAR_DISK_STATUS, DEF_DISK_OPEN_DIR);
#endif
    xWriteCH376Cmd(CMD2H_BYTE_WRITE);
    xWriteCH376Data((UINT8)ReqCount);
    xWriteCH376Data((UINT8)(ReqCount >> 8));
    xEndCH376Cmd();
    while (1) {
        s = Wait376Interrupt();
        if (s == USB_INT_DISK_WRITE) {
            if (buf)
                buf += CH376WriteReqBlock(buf); /* Write the requested data block to the internal designated buffer and return the length */
            else {
                xWriteCH376Cmd(CMD01_WR_REQ_DATA); /* Write the requested data block to the internal specified buffer */
                s = xReadCH376Data();              /* Length */
                while (s--)
                    xWriteCH376Data(0); /* Fill with 0 */
            }
            xWriteCH376Cmd(CMD0H_BYTE_WR_GO);
            xEndCH376Cmd();
        }
        /*		else if ( s == USB_INT_SUCCESS ) return( s );*/ /* Finish */
        else {
#ifndef DEF_IC_V43_U
            if (c == DEF_DISK_OPEN_ROOT)
                CH376WriteVar8(VAR_DISK_STATUS, c);
#endif
            return (s); /* Error */
        }
    }
}

UINT8 CH376CheckNameSum(PUINT8 DirName) /* Computes the short filename checksum of a long filename, input in a fixed 11-byte format with no decimal separator */
{
    UINT8 NameLen;
    UINT8 CheckSum;
    CheckSum = 0;
    for (NameLen = 0; NameLen != 11; NameLen++)
        CheckSum = (CheckSum & 1 ? 0x80 : 0x00) + (CheckSum >> 1) + *DirName++;
    return (CheckSum);
}

UINT8 CH376LocateInUpDir(PUINT8 PathName) /* Move the file pointer in the parent directory (folder) to the sector where the directory information of the current file is located */
/* In addition, by the way, write the LBA address of the previous sector of the sector where the current file directory information is located into the CH376 internal VAR_FAT_DIR_LBA variable (for the convenience of searching forward when collecting long file names, otherwise it needs to be moved once more)
 */
/* The first 12 bytes of the global buffer GlobalBuf are used */
{
    UINT8 s;
    xWriteCH376Cmd(CMD14_READ_VAR32);
    xWriteCH376Data(VAR_FAT_DIR_LBA); /* The LBA address of the sector where the current file directory information is located */
    for (s = 4; s != 8; s++)
        GlobalBuf[s] = xReadCH376Data(); /* Temporarily save in the global buffer to save RAM */
    xEndCH376Cmd();
    s = CH376SeparatePath(PathName); /* Separate the last-level file name or directory name from the path, and return the offset of the last-level file name or directory name */
    if (s)
        s = CH376FileOpenDir(PathName, s); /* It is a multi-level directory, open the last level directory under the multi-level directory, that is, open the upper-level directory of the file */
    else
        s = CH376FileOpen("/"); /* Files in the root directory, open the root directory */
    if (s != ERR_OPEN_DIR)
        return (s);
    *(PUINT32)(&GlobalBuf[0]) = 0; /* Directory sector offset sector number, stored in the global buffer, saving RAM */
    while (1) { /* Keep moving the file pointer until it matches the LBA address of the sector where the current file directory information is located */
        s = CH376SecLocate(*(PUINT32)(&GlobalBuf[0])); /* Move the file pointer in the parent directory in units of sectors */
        if (s != USB_INT_SUCCESS)
            return (s);
        CH376ReadBlock(&GlobalBuf[8]); /* Read CH376_CMD_DATA.SectorLocate.mSectorLba data block from memory buffer, return length is always sizeof(CH376_CMD_DATA.SectorLocate) */
        if (*(PUINT32)(&GlobalBuf[8]) == *(PUINT32)(&GlobalBuf[4]))
            return (USB_INT_SUCCESS); /* The directory information sector of the current file has been reached */
        xWriteCH376Cmd(CMD50_WRITE_VAR32);
        xWriteCH376Data(VAR_FAT_DIR_LBA); /* Get the previous sector and set it as the LBA address of the new file directory information sector */
        for (s = 8; s != 12; s++)
            xWriteCH376Data(GlobalBuf[s]);
        xEndCH376Cmd();
        ++*(PUINT32)(&GlobalBuf[0]);
    }
}

UINT8 CH376GetLongName(PUINT8 PathName, PUINT8 LongName) /* Obtain the corresponding long file name from the short file name or directory (folder) name */
/* You need to enter the full path PathName of the short file name, and you need to provide a buffer to receive the long file name LongName (encoded in UNICODE little endian, ending with double 0) */
/* The first 34 bytes of the global buffer GlobalBuf are used, sizeof(GlobalBuf)>=sizeof(FAT_DIR_INFO)+2 */
{
    UINT8 s;
    UINT16 NameCount;                /* Long filename byte count */
    s = CH376FileOpenPath(PathName); /* Open a file or directory under a multi-level directory*/
    if (s != USB_INT_SUCCESS && s != ERR_OPEN_DIR)
        return (s);
    s = CH376DirInfoRead(); /* Read the directory information FAT_DIR_INFO of the current file, and transfer the relevant data to the memory */
    if (s != USB_INT_SUCCESS)
        return (s);
    CH376ReadBlock(GlobalBuf); /* Read the FAT_DIR_INFO data block from the memory buffer, the return length is always sizeof(FAT_DIR_INFO) */
    CH376EndDirInfo();         /* After getting the FAT_DIR_INFO structure */
    GlobalBuf[32] = CH376CheckNameSum(GlobalBuf); /* Calculate the checksum of the short file name of the long file name and store it in the global buffer to save RAM*/
    GlobalBuf[33] = CH376ReadVar8(VAR_FILE_DIR_INDEX); /* The index number of the current file directory information in the sector is stored in the global buffer to save RAM*/
    NameCount     = 0;
    while (1) {
        if (GlobalBuf[33] == 0) {             /* The processing of the current file directory information sector is completed, and it goes to the previous sector */
            s = CH376LocateInUpDir(PathName); /* Move the file pointer in the upper-level directory to the sector where the directory information of the current file is located */
            if (s != USB_INT_SUCCESS)
                break;
            if (CH376ReadVar32(VAR_CURRENT_OFFSET) == 0) { /* The current is already at the beginning of the directory sector, unable to get the long file name */
                s = ERR_LONG_NAME_ERR;
                break;
            }
            GlobalBuf[33] = DEF_SECTOR_SIZE / sizeof(FAT_DIR_INFO); /* Point to the last file directory information of the previous sector */
        }
        GlobalBuf[33]--; /* Search file directory information from back to front */
        s = CH376SendCmdDatWaitInt(
            CMD1H_DIR_INFO_READ, GlobalBuf[33]); /* Read the specified directory information FAT_DIR_INFO, and transfer the relevant data to the memory */
        if (s != USB_INT_SUCCESS)
            break;
        CH376ReadBlock(GlobalBuf); /* Read the FAT_DIR_INFO data block from the memory buffer, the return length is always sizeof(FAT_DIR_INFO)*/
        CH376EndDirInfo();         /* Getting all the FAT_DIR_INFO structure */
        if ((GlobalBuf[11] & ATTR_LONG_NAME_MASK) != ATTR_LONG_NAME ||
            GlobalBuf[13] != GlobalBuf[32]) { /* Type error or checksum error */
            s = ERR_LONG_NAME_ERR;
            break; /* There is no direct return because if the root directory is opened, it must be closed before returning */
        }
        for (s = 1; s < sizeof(FAT_DIR_INFO); s += 2) { /* Collect long file names, the characters of long file names are stored in UNICODE on the disk in little endian mode */
            if (s == 1 + 5 * 2)
                s = 14; /* Jump from the first set of 1-5 characters to the second set of 6-11 characters in long filenames */
            else if (s == 14 + 6 * 2)
                s = 28; /* Jump from the second set of 6-11 characters to the third set of 12-13 characters in long filenames */
            LongName[NameCount++] = GlobalBuf[s];
            LongName[NameCount++] = GlobalBuf[s + 1];
            if (GlobalBuf[s] == 0 && GlobalBuf[s + 1] == 0)
                break;                            /* End of long filename */
            if (NameCount >= LONG_NAME_BUF_LEN) { /* Long filename buffer overflow */
                s = ERR_LONG_BUF_OVER;
                goto CH376GetLongNameE;
            }
        }
        if (GlobalBuf[0] & 0x40) { /* End of long filename directory information block */
            if (s >= sizeof(FAT_DIR_INFO))
                *(PUINT16)(&LongName[NameCount]) = 0x0000; /* If the end character of the long file name has not been collected, it will be forced to end */
            s = USB_INT_SUCCESS;                           /* Successful completion of long filename collection complete */
            break;
        }
    }
CH376GetLongNameE:
    CH376FileClose(FALSE); /* For the root directory, it must be closed */
    return (s);
}

UINT8 CH376CreateLongName(
    PUINT8 PathName, PUINT8 LongName) /* LongName输入路径必须在RAM中,Create a new file with a long file name, return after closing the file, the LongName input path must be in RAM */
/* You need to enter the full path PathName of the short file name (please refer to the FAT specification to generate the long file name yourself), and you need to enter the long file name LongName encoded in UNICODE little endian and ending with double 0
 */
/* The first 64 bytes of the global buffer GlobalBuf are used, sizeof(GlobalBuf)>=sizeof(FAT_DIR_INFO)*2 */
{
    UINT8 s, i;
    UINT8 DirBlockCnt; /* The number of long file names occupying the file directory structure */
    UINT16 count;      /* Temporary variable, used for counting, used for the actual number of bytes read in the byte read file mode */
    UINT16 NameCount;  /* Long filename byte count */
    UINT32 NewFileLoc; /* The starting position and offset address of the current file directory information in the parent directory */
    for (count = 0; count < LONG_NAME_BUF_LEN; count += 2)
        if (*(PUINT16)(&LongName[count]) == 0)
            break; /* To end position */
    if (count == 0 || count >= LONG_NAME_BUF_LEN || count > LONE_NAME_MAX_CHAR)
        return (ERR_LONG_NAME_ERR);          /* Invalid long filename */
    DirBlockCnt = count / LONG_NAME_PER_DIR; /* The number of long file names occupying the file directory structure */
    i           = count - DirBlockCnt * LONG_NAME_PER_DIR;
    if (i) { /* There is a fraction */
        if (++DirBlockCnt * LONG_NAME_PER_DIR > LONG_NAME_BUF_LEN)
            return (ERR_LONG_BUF_OVER); /* Buffer overflow */
        count += 2;                     /* The length after adding the 0 terminator */
        i += 2;
        if (i < LONG_NAME_PER_DIR) { /* The final file directory structure is not satisfied */
            while (i++ < LONG_NAME_PER_DIR)
                LongName[count++] = 0xFF; /* Fill the remaining data with 0xFF */
        }
    }
    s = CH376FileOpenPath(PathName); /* Open files in multi-level directories */
    if (s == USB_INT_SUCCESS) {      /* Returns an error if the short filename exists */
        s = ERR_NAME_EXIST;
        goto CH376CreateLongNameE;
    }
    if (s != ERR_MISS_FILE)
        return (s);
    s = CH376FileCreatePath(PathName); /* Create new files in multi-level directories */
    if (s != USB_INT_SUCCESS)
        return (s);
    i = CH376ReadVar8(VAR_FILE_DIR_INDEX); /* Temporarily used to save the index number of the current file directory information in the sector */
    s = CH376LocateInUpDir(PathName); /* Move the file pointer in the upper-level directory to the sector where the directory information of the current file is located */
    if (s != USB_INT_SUCCESS)
        goto CH376CreateLongNameE; /* There is no direct return because if the root directory is opened, it must be closed before returning */
    NewFileLoc = CH376ReadVar32(VAR_CURRENT_OFFSET) +
        i * sizeof(FAT_DIR_INFO);    /* Calculate the starting position and offset address of the current file directory information in the upper-level directory */
    s = CH376ByteLocate(NewFileLoc); /* Move the file pointer to the location of the directory information of the current file in the parent directory */
    if (s != USB_INT_SUCCESS)
        goto CH376CreateLongNameE;
    s = CH376ByteRead(&GlobalBuf[sizeof(FAT_DIR_INFO)],
        sizeof(FAT_DIR_INFO),
        NULL); /* Read data in bytes and get the directory information of the current file FAT_DIR_INFO */
    if (s != USB_INT_SUCCESS)
        goto CH376CreateLongNameE;
    for (i = DirBlockCnt; i != 0; --i) { /* Search for free file directory structures for long filenames */
        s = CH376ByteRead(
            GlobalBuf, sizeof(FAT_DIR_INFO), &count); /* Read data in bytes to get the next file directory information FAT_DIR_INFO */
        if (s != USB_INT_SUCCESS)
            goto CH376CreateLongNameE;
        if (count == 0)
            break;                                  /* Unable to read data, parent directory ended */
        if (GlobalBuf[0] && GlobalBuf[0] != 0xE5) { /* There is a file directory structure in use in the back, because the long file name must be connected and stored, so there is not enough space, the current location must be abandoned and moved backwards
                                                     */
            s = CH376ByteLocate(NewFileLoc); /* Move the file pointer to the location of the directory information of the current file in the parent directory */
            if (s != USB_INT_SUCCESS)
                goto CH376CreateLongNameE;
            GlobalBuf[0] = 0xE5; /* File delete flag */
            for (s = 1; s != sizeof(FAT_DIR_INFO); s++)
                GlobalBuf[s] = GlobalBuf[sizeof(FAT_DIR_INFO) + s];
            s = CH376LongNameWrite(GlobalBuf, sizeof(FAT_DIR_INFO)); /* Write a file directory structure, used to delete previously created files, which will actually be transferred to the end of the directory later
                                                                      */
            if (s != USB_INT_SUCCESS)
                goto CH376CreateLongNameE;
            do { /* Search backwards through the free file directory structure */
                s = CH376ByteRead(GlobalBuf,
                    sizeof(FAT_DIR_INFO),
                    &count); /* Read data in bytes to get the next file directory information FAT_DIR_INFO*/
                if (s != USB_INT_SUCCESS)
                    goto CH376CreateLongNameE;
            } while (count && GlobalBuf[0]); /* If the file directory structure is still in use, continue to search backward until the end of the upper directory or there is an unused file directory structure
                                              */
            NewFileLoc = CH376ReadVar32(
                VAR_CURRENT_OFFSET); /* Use the current file pointer of the upper-level directory as the starting position of the current file directory information in the upper-level directory */
            i = DirBlockCnt + 1; /* The number of free file directory structures required, including the short file name itself and the long file name */
            if (count == 0)
                break;                          /* Unable to read data, parent directory ended */
            NewFileLoc -= sizeof(FAT_DIR_INFO); /* Rewind to the beginning of the free file directory structure just searched */
        }
    }
    if (i) { /* The free file directory structure is not enough to store long file names, the reason is that the upper-level directory is over, and the length of the upper-level directory is increased below */
        s = CH376ReadVar8(VAR_SEC_PER_CLUS); /* Sectors per cluster */
        if (s == 128) {       /* The root directory of FAT12/FAT16 has a fixed capacity and cannot increase the file directory structure */
            s = ERR_FDT_OVER; /* The number of files in the FAT12/FAT16 root directory should be less than 512, and disk defragmentation is required */
            goto CH376CreateLongNameE;
        }
        count = s * DEF_SECTOR_SIZE; /* Bytes per cluster */
        if (count < i * sizeof(FAT_DIR_INFO))
            count <<= 1; /* If one cluster is not enough, add another cluster, this will only happen when each cluster is 512 bytes */
        s = CH376LongNameWrite(NULL, count); /* Write all 0 data blocks to the current location in bytes, and clear the newly added file directory cluster */
        if (s != USB_INT_SUCCESS)
            goto CH376CreateLongNameE;
    }
    s = CH376ByteLocate(NewFileLoc); /* Move the file pointer to the location of the directory information of the current file in the parent directory */
    if (s != USB_INT_SUCCESS)
        goto CH376CreateLongNameE;
    GlobalBuf[11] = ATTR_LONG_NAME;
    GlobalBuf[12] = 0x00;
    GlobalBuf[13] = CH376CheckNameSum(&GlobalBuf[sizeof(FAT_DIR_INFO)]); /* Calculate short filename checksum of long filename */
    GlobalBuf[26] = 0x00;
    GlobalBuf[27] = 0x00;
    for (s = 0; DirBlockCnt != 0;) {                         /* Count of file directory structures occupied by long filenames */
        GlobalBuf[0] = s ? DirBlockCnt : DirBlockCnt | 0x40; /* The first time to set the long file name entry flag */
        DirBlockCnt--;
        NameCount = DirBlockCnt * LONG_NAME_PER_DIR;
        for (s = 1; s < sizeof(FAT_DIR_INFO); s += 2) { /* Output the long file name, the characters of the long file name are stored in UNICODE on the disk in little endian mode */
            if (s == 1 + 5 * 2)
                s = 14; /* Jump from the first set of 1-5 characters to the second set of 6-11 characters in long filenames */
            else if (s == 14 + 6 * 2)
                s = 28; /* Jump from the second set of 6-11 characters to the third set of 12-13 characters in long filenames */
            GlobalBuf[s]     = LongName[NameCount++];
            GlobalBuf[s + 1] = LongName[NameCount++];
        }
        s = CH376LongNameWrite(GlobalBuf, sizeof(FAT_DIR_INFO)); /* Write a file directory structure in bytes, long file name */
        if (s != USB_INT_SUCCESS)
            goto CH376CreateLongNameE;
    }
    s = CH376LongNameWrite(&GlobalBuf[sizeof(FAT_DIR_INFO)],
        sizeof(FAT_DIR_INFO)); /* Write a file directory structure in bytes, which is the directory information of the previously created file transferred */
CH376CreateLongNameE:
    CH376FileClose(FALSE); /* For the root directory, it must be closed */
    return (s);
}

#endif